unit rPatients;

interface

uses SysUtils, Classes, ORNet, ORFn, uCohort;

{ Patient List calls }

function DfltPtList: string;
function DfltPtListSrc: Char;

procedure ListSpecialtyAll(Dest: TStrings);
procedure ListTeamAll(Dest: TStrings);
procedure ListWardAll(Dest: TStrings);
procedure ListRegistryAll(Dest: TStrings);

function SubSetOfProviders(const StartFrom: string; Direction: Integer): TStrings;
function SubSetOfClinics(const StartFrom: string; Direction: Integer): TStrings;

procedure ListPtByProvider(Dest: TStrings; ProviderIEN: Int64);
procedure ListPtByTeam(Dest: TStrings; TeamIEN: Int64);
procedure ListPtBySpecialty(Dest: TStrings; SpecialtyIEN: Int64);
procedure ListPtByClinic(Dest: TStrings; ClinicIEN: Int64; FirstDt, LastDt: string); //TFMDateTime); Modified by PKS - 4/9/2001.
procedure ListPtByWard(Dest: TStrings; WardIEN: Int64); // Modified by PKS - 4/9/2001.
procedure ListPtByRegistry(Dest: TStrings; const RegName, Mode: string);

function ExternalName(IEN: Int64; FileNumber: Double): string; overload;
function ExternalName(IEN: String; FileNumber: Double): string; overload;
function RegistryName(const AnID: string): string;

function HasKey(const AKey: string): Boolean;

implementation

{ Patient List Calls }

function DfltPtList: string;
{ returns the name of the current user's default patient list, null if none is defined
  Pieces: Ptr to Source File^Source Name^Source Type }
begin
  Result := sCallV('ORQPT DEFAULT LIST SOURCE', [nil]);
  if Length(Result) > 0 then Result := Piece(Result, '^', 2);
end;

function DfltPtListSrc: Char;
begin
  Result := CharAt(sCallV('ORWPT DFLTSRC', [nil]), 1);
end;

procedure ListSpecialtyAll(Dest: TStrings);
{ lists all treating specialties: IEN^Treating Specialty Name }
begin
  CallV('ORQPT SPECIALTIES', [nil]);
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListTeamAll(Dest: TStrings);
{ lists all patient care teams: IEN^Team Name }
begin
  CallV('ORQPT TEAMS', [nil]);
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListWardAll(Dest: TStrings);
{ lists all active inpatient wards: IEN^Ward Name }
begin
  CallV('ORQPT WARDS', [nil]);
  //MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;


procedure ListRegistryAll(Dest: TStrings);
{ lists all available patient registeries }
begin
  CallV('ORRHCU REGLST', [nil]);
  Dest.Assign(RPCBrokerV.Results);
end;

function SubSetOfProviders(const StartFrom: string; Direction: Integer): TStrings;
{ returns a pointer to a list of providers (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('ORWU NEWPERS', [StartFrom, Direction, 'PROVIDER']);
  MixedCaseList(RPCBrokerV.Results);
  Result := RPCBrokerV.Results;
end;

function SubSetOfClinics(const StartFrom: string; Direction: Integer): TStrings;
{ returns a pointer to a list of clinics (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('ORWU CLINLOC', [StartFrom, Direction]);
  MixedCaseList(RPCBrokerV.Results);
  Result := RPCBrokerV.Results;
end;

procedure ListPtByProvider(Dest: TStrings; ProviderIEN: Int64);
{ lists all patients associated with a given provider: DFN^Patient Name }
begin
  CallV('ORQPT PROVIDER PATIENTS', [ProviderIEN]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2);  // alpha sort
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListPtByTeam(Dest: TStrings; TeamIEN: Int64);
{ lists all patients associated with a given team: DFN^Patient Name }
begin
  CallV('ORQPT TEAM PATIENTS', [TeamIEN]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2);  // alpha sort
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListPtBySpecialty(Dest: TStrings; SpecialtyIEN: Int64);
{ lists all patients associated with a given specialty: DFN^Patient Name }
begin
  CallV('ORQPT SPECIALTY PATIENTS', [SpecialtyIEN]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2); // alpha sort
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListPtByClinic(Dest: TStrings; ClinicIEN: Int64; FirstDt, LastDt: string); //TFMDateTime);
{ lists all patients associated with a given clinic: DFN^Patient Name^App't }
begin
  CallV('ORQPT CLINIC PATIENTS', [ClinicIEN, FirstDt, LastDt]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2); // alpha sort
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListPtByWard(Dest: TStrings; WardIEN: Int64);
{ lists all patients associated with a given ward: DFN^Patient Name^Room/Bed }
begin
  CallV('ORWPT BYWARD', [WardIEN]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2);
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure ListPtByRegistry(Dest: TStrings; const RegName, Mode: string);
begin
  CallV('ORRHCU BYREG', [RegName, Mode]);
  SortByPiece(TStringList(RPCBrokerV.Results), U, 2);
  MixedCaseList(RPCBrokerV.Results);
  Dest.Assign(RPCBrokerV.Results);
end;

{ General calls }

function ExternalName(IEN: Int64; FileNumber: Double): string; overload;
{ returns the external name of the IEN within a file }
begin
  Result := sCallV('ORWU EXTNAME', [IEN, FileNumber]);
end;

function ExternalName(IEN: String; FileNumber: Double): string; overload;
{ returns the external name of the IEN within a file }
begin
  Result := sCallV('ORWU EXTNAME', [IEN, FileNumber]);
end;

function RegistryName(const AnID: string): string;
begin
  Result := sCallV('ORRHCU REGNAM', [AnID]);
end;

function HasKey(const AKey: string): Boolean;
begin
  Result := sCallV('ORWU HASKEY', [AKey]) = '1';
end;

end.
