unit rReport;

interface

uses SysUtils, Classes, ORNet, ORFn;

function SubSetOfClinics(const StartFrom: string; Direction: Integer): TStrings;
function SubSetOfPersons(const StartFrom: string; Direction: Integer): TStrings;
function SubSetOfDocuments(const StartFrom: string; Direction, DocClass: Integer): TStrings;
function SubSetOfOrderables(const StartFrom: string; Direction: Integer; ASet: string): TStrings;
procedure LoadExternalValues(ValueList: TStrings; const FilemanFile: string);
procedure LoadOrderableItemFilters(Dest: TStrings);
procedure LoadDocumentClasses(Dest: TStrings);
procedure ListDGroupAll(Dest: TStrings);
procedure LoadReportList(Dest: TStrings);
procedure LoadOrderStatuses(Dest: TStrings);
procedure LoadOrderSignStatuses(Dest: TStrings);
procedure LoadDocumentStatuses(Dest: TStrings);
procedure LoadCriteriaTree(Dest: TStrings);
procedure LoadDisplayFields(Dest: TStrings; const FieldTypes: string);
procedure LoadReportDefinition(Dest: TStrings; AQueryID: Integer);
procedure LoadReportDefinitionTag(Dest: TStrings; ATag: Integer);
procedure LoadQueryData(Dest: TStrings; First, Last: Integer);
function SetupQuery(QueryStrings: TStringList): string;
function QueryIterate(const AnIterator: string): string;
function DoQueryBySequence(First, Last: Integer): Integer;
function ApplyPatientScreen(ADFNList: TStringList): Integer;
function SortByFieldName(const AFieldName: string; Ascending: Boolean): Integer;
function SaveReportDefinition(TextDef: TStrings): string;
function GetReportIdForUser(const ARPTName: string): Integer;
function GetCousultGrp: string;
procedure DetailOrder(Dest: TStrings; const AnID: string);
procedure DetailDocument(Dest: TStrings; const AnID: string);
function GetPtInfo(const AnID: string): string;
procedure DeleteReport(AnIEN: Integer);
function ReportOwnedByUser(AnIEN: Integer): Boolean;
procedure LoadColumnTypes(ColTypeList: TStringList);
function StartDateForAbnormal: string;
function FMDateRange(DTrange: string): string;
procedure GenericReportList(var AList: TStringList);
procedure ClearQuery;

function ValuePart(const x: string): string;

implementation

function SubSetOfClinics(const StartFrom: string; Direction: Integer): TStrings;
{ returns a pointer to a list of clinics (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('ORWU CLINLOC', [StartFrom, Direction]);
  //MixedCaseList(RPCBrokerV.Results);
  Result := RPCBrokerV.Results;
end;

function SubSetOfDocuments(const StartFrom: string; Direction, DocClass: Integer): TStrings;
{ returns a pointer to a list of titles (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('TIU LONG LIST OF TITLES', [DocClass, StartFrom, Direction]);
  Result := RPCBrokerV.Results;
end;

function SubSetOfPersons(const StartFrom: string; Direction: Integer): TStrings;
{ returns a pointer to a list of persons (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  CallV('ORWU NEWPERS', [StartFrom, Direction]);
  Result := RPCBrokerV.Results;
end;

function SubSetOfOrderables(const StartFrom: string; Direction: Integer; ASet: string): TStrings;
{ returns a pointer to a list of orderable items (for use in a long list box) -  The return value is
  a pointer to RPCBrokerV.Results, so the data must be used BEFORE the next broker call! }
begin
  if ASet = '' then ASet := 'B' else ASet := 'S.' + ASet;
  CallV('ORRHCO ORDITM', [StartFrom, Direction, ASet]);
  Result := RPCBrokerV.Results;
end;

procedure LoadExternalValues(ValueList: TStrings; const FilemanFile: string);
begin
  if ValueList.Count > 0 then
  begin
    CallV('ORRHCU ID2EXT', [FilemanFile, ValueList]);
    ValueList.Assign(RPCBrokerV.Results);
  end;
end;

procedure LoadOrderableItemFilters(Dest: TStrings);
begin
  CallV('ORRHCO OISETS', [nil]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadDocumentClasses(Dest: TStrings);
var
  TempList: TStringList;
begin
  CallV('ORRHCT DOCCLS', [nil]);
  TempList := TStringList.Create;
  TempList.Assign(RPCBrokerV.Results);
  SortByPiece(TempList, '^', 2);
  Dest.Assign(TempList);
  TempList.Free;
end;

procedure ListDGroupAll(Dest: TStrings);
begin
  CallV('ORWORDG ALLTREE', [nil]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadReportList(Dest: TStrings);
begin
  CallV('ORRHCR RPTLST', []);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadOrderStatuses(Dest: TStrings);
begin
  CallV('ORRHCO ORDSTS', []);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadOrderSignStatuses(Dest: TStrings);
begin
  CallV('ORRHCO SIGNSTS', []);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadDocumentStatuses(Dest: TStrings);
begin
  CallV('ORRHCT DOCSTS', []);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadCriteriaTree(Dest: TStrings);
begin
  CallV('ORRHCR CTPLST', []);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadDisplayFields(Dest: TStrings; const FieldTypes: string);
begin
  CallV('ORRHCU DFLDS', [FieldTypes]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadReportDefinition(Dest: TStrings; AQueryID: Integer);
begin
  CallV('ORRHCR RPTDEF', [AQueryID]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadReportDefinitionTag(Dest: TStrings; ATag: Integer);
begin
  CallV('ORRHCR TAGDEF', [ATag]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure LoadQueryData(Dest: TStrings; First, Last: Integer);
begin
  // M side is not zero based, so First & Last must be incremented
  CallV('ORRHCQ SUBDTA', [First+1, Last+1]);
  Dest.Assign(RPCBrokerV.Results);
end;

function SetupQuery(QueryStrings: TStringList): string;
begin
  Result := '';
  CallV('ORRHCQ SETUP', [QueryStrings]);
  if RPCBrokerV.Results.Count > 0 then Result := RPCBrokerV.Results[0];
end;

function QueryIterate(const AnIterator: string): string;
begin
  Result := '';
  CallV('ORRHCQ QRYITR', [AnIterator]);
  if RPCBrokerV.Results.Count > 0 then Result := RPCBrokerV.Results[0];
end;

function DoQueryBySequence(First, Last: Integer): Integer;
begin
  Result := 0;
  CallV('ORRHCQ QRYSEQ', [First, Last]);
  if RPCBrokerV.Results.Count > 0 then Result := StrToInt(RPCBrokerV.Results[0]);
end;

function ApplyPatientScreen(ADFNList: TStringList): Integer;
begin
  Result := 0;
  CallV('ORRHCQ FILTER', [ADFNList]);
  if RPCBrokerV.Results.Count > 0 then Result := StrToInt(RPCBrokerV.Results[0]);
end;

function SortByFieldName(const AFieldName: string; Ascending: Boolean): Integer;
begin
  Result := 0;
  CallV('ORRHCQ SORTBY', [AFieldName, Ascending]);
  if RPCBrokerV.Results.Count > 0 then Result := StrToInt(RPCBrokerV.Results[0]);
end;

function SaveReportDefinition(TextDef: TStrings): string;
begin
  CallV('ORRHCR SAVDEF', [TextDef]);
  if RPCBrokerV.Results.Count > 0
    then Result := RPCBrokerV.Results[0]
    else Result := '';
end;

function GetReportIdForUser(const ARPTName: string): Integer;
begin
  CallV('ORRHCR USRRPT', [ARPTName]);
  if RPCBrokerV.Results.Count > 0
    then Result := StrToIntDef(RPCBrokerV.Results[0], 0)
    else Result := 0;
end;

function GetCousultGrp: string;
begin
  Result := SCallV('ORRHCO CSLTGRP',[nil]);
end;

procedure DetailOrder(Dest: TStrings; const AnID: string);
begin
  CallV('ORRHCQ DETAIL', [AnID]);
  Dest.Assign(RPCBrokerV.Results);
end;

procedure DetailDocument(Dest: TStrings; const AnID: string);
{ returns the text of a document (progress note, discharge summary, etc.) }
begin
  CallV('TIU GET RECORD TEXT', [AnID]);
  Dest.Assign(RPCBrokerV.Results);
end;

function GetPtInfo(const AnID: string): string;
begin
  Result := '';
  CallV('ORRHCQ PTINFO', [AnID]);
  if RPCBrokerV.Results.Count > 0 then Result := RPCBrokerV.Results[0];
end;

procedure DeleteReport(AnIEN: Integer);
begin
  CallV('ORRHCR DELETE', [AnIEN]);
end;

function ReportOwnedByUser(AnIEN: Integer): Boolean;
begin
  Result := False;
  CallV('ORRHCR OWNED', [AnIEN]);
  if RPCBrokerV.Results.Count > 0 then Result := RPCBrokerV.Results[0] = '1';
end;

procedure LoadColumnTypes(ColTypeList: TStringList);
begin
  CallV('ORRHCU COLTYP', [ColTypeList]);
  ColTypeList.Assign(RPCBrokerV.Results);
end;

function StartDateForAbnormal: string;
begin
  Result := SCallV('ORRHCO ABSTRT',[nil]);
end;

function FMDateRange(DTrange: string): string;
begin
  Result := SCallV('ORRHCQ RNGFM',[DTrange]);
end;

procedure GenericReportList(var AList: TStringList);
begin
  CallV('ORRHCR GENRPT',[nil]);
  AList.Assign(RPCBrokerV.Results);
end;

procedure ClearQuery;
begin
  CallV('ORRHCQ CLEAR', [nil]);
end;

{ General calls }

function ValuePart(const x: string): string;
begin
  Result := Copy(x, Pos('=', x) + 1, Length(x));
end;

end.
