unit uReport;

interface

uses SysUtils, Classes, Forms, ComCtrls, Dialogs, Windows, Controls;


const
  EDITOR_DATE =  2;
  EDITOR_NONE = 12;
  PT_LIST                   = 1000500;
  PT_SELECTED_VISIT         = 1010500;
  PT_ANY_VISIT              = 1011000;
  PT_PRI_PROVIDER           = 1011500;
  ITEM_DATERANGE            = 1500500;
  RPT_ORDERS                = 2000500;
  RPT_CONSULTS              = 3000500;
  RPT_DOCUMENTS             = 4000500;
  RPT_VISITS                = 5000500;

{ TODO -oLori -cHepC : TQueryText Class }
type
  TQueryText = class
  private
    FCaption: string;
    FConstraintName: string;
    FCriterionTag: Integer;
    FEditorId: Integer;
    FSequence: Integer;
    FValueList: TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    property Caption: string        read FCaption         write FCaption;
    property ConstraintName: string read FConstraintName;
    property CriterionTag: Integer  read FCriterionTag;
    property EditorId: Integer      read FEditorID;
    property Values: TStringList    read FValueList       write FValueList;
  end;

  TReport = class
  private
    FId: Integer;
    FName: string;
    FDisplayText: string;
    FParents: TStringList;
    FColList: TList;
    FQueryList: TList;
    FQueryText: TQueryText;
    function GetColCount: Integer;
    function GetColHeader(Index: Integer): string;
    function GetColName(Index: Integer): string;
    function GetColWidth(Index: Integer): Integer;
    function FindSequenceBeforeDateRangeQuery: Integer;
    procedure SetColCount(AValue: Integer);
    procedure SetColHeader(Index: Integer; AValue: string);
    procedure SetColName(Index: Integer; const Value: string);
    procedure SetColWidth(Index, AValue: Integer);
    function GetQueryText(Index: Integer): TQueryText;
    procedure LoadFormatting(Dest: TList; Src: TStringList);
    procedure LoadQueryText(Dest: TList; Src: TStringList);
    procedure SetDisplayText(const AValue: string);
    procedure SetID(Value: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    procedure AddCriterion(ATag: Integer; FollowPtList: boolean = False);
    procedure RemoveCriterion(ATag: Integer; UpdateSeq: boolean = False);
    function ContainsConstraint(AName: string): Boolean;
    function ContainsCriterion(ATag: Integer): Boolean;
    procedure Delete;
    procedure DefinitionToStrings(Dest: TStrings);
    function ExistsOnServer: Boolean;
    procedure FillNameValueList(Dest: TStrings);
    function IsOwner: Boolean;
    procedure ListNamesByUser(Dest: TStrings);
    procedure QueryToStrings(Dest: TStrings);
    procedure Save(ToServer: Boolean);
    function ValuesAreMissing: Boolean;
    procedure ValuesByConstraint(AValueList: TStrings; const AConstraint: string);
    property ColCount: Integer                     read GetColCount  write SetColCount;
    property ColHeader[Index: Integer]: string     read GetColHeader write SetColHeader;
    property ColName[Index: Integer]: string       read GetColName   write SetColName;
    property ColWidth[Index: Integer]: Integer     read GetColWidth  write SetColWidth;
    property DisplayText: string                   read FDisplayText write SetDisplayText;
    property Id: Integer                           read FId          write SetID;
    property Name: string                          read FName        write FName;
    property QueryText[Index: Integer]: TQueryText read GetQueryText;
    property QueryList: TList                      read FQueryList   write FQueryList;
    property ColList: TList                        read FColList     write FColList;
  end;

  procedure SaveEditedReport;
  function IsGenericReport(ARPTDisplayName: string): boolean;

var
  Report: TReport = nil;
  OldQuery: TStringList;

implementation

uses ORFn, rDateTime, rReport;

type
  TColFormat = class
  public
    Name: string;
    Header: string;
    Width: Integer;
  end;

  TReportCache = class
  private
    FNextIEN: Integer;
    FCache: TList;
  public
    constructor Create;
    destructor Destroy; override;
    function GetNextIEN: Integer;
    procedure FillCachedReportList(Dest: TStrings);
    procedure SaveToCache(ATextDef: TStringList);
    procedure LoadFromCache(ATextDef: TStringList; AnIEN: Integer);
    procedure RemoveFromCache(AnIEN: Integer);
  end;

var
  ReportCache: TReportCache = nil;

{ General Functions ------------------------------------------------------------------------ }

function RelativeDateToCaption(const ADate: string): string;
{ create a caption based on the current date since dates are always stored as relative date/times }
var
  YearType, Quarter, Date1, Date2, x1, x2: string;
  CurrentYear, CurrentMonth, OffSet: Integer;
begin
  Result := '';
  if ADate = 'T:T'    then Result := 'during Today';
  if ADate = 'T-1:T'  then Result := 'during the Past 24 Hours';
  if ADate = 'T-7:T'  then Result := 'during the Past Week';
  if ADate = 'T-30:T' then Result := 'during the Past Month';
  if ADate = 'YFQ'    then Result := 'during the Current Quarter';
  if ADate = 'YF'     then Result := 'during the Current Year';
  if (Result = '') and (CharAt(ADate, 1) = 'Y') then
  begin
    Result := 'during ';
    Quarter := Piece(ADate, 'Q', 2);
    case CharAt(Quarter, 1) of
    '1': Result := Result + '1st Quarter ';
    '2': Result := Result + '2nd Quarter ';
    '3': Result := Result + '3rd Quarter ';
    '4': Result := Result + '4th Quarter ';
    end;
    if CharAt(ADate, 2) = 'F' then YearType := 'FY ' else YearType := '';
    x1 := Piece(ADate, 'Q', 1);
    x2 := Copy(x1, 3, Length(x1));
    Offset := StrToIntDef(x2, 0);
    CurrentMonth := (Trunc(FMToday) mod 10000) div 100;
    CurrentYear  := Trunc(FMToday / 10000) + 1700;
    if (CharAt(ADate, 2) = 'F') and (CurrentMonth > 9) then Inc(CurrentYear);
    Result := Result + YearType + IntToStr(CurrentYear + Offset);
  end;
  if (Result = '') and (Pos(':', ADate) > 0) then
  begin
    Result := '';
    Date1 := Piece(ADate, ':', 1);
    Date2 := Piece(ADate, ':', 2);
    if Length(Date1) > 0
      then Result := Result + 'from '     + FormatFMDateTime('dd mmm yyyy', StrToFMDateTime(Date1));
    if Length(Date2) > 0
      then Result := Result + ' through ' + FormatFMDateTime('dd mmm yyyy', StrToFMDateTime(Date2));
    if (ADate = ':') or (ADate = '0:0') then Result := 'any date';
  end;
  if Result = '' then Result := '< select date range >';
end;

{ TQueryText ------------------------------------------------------------------------------- }

{ TQueryText constructors and destructors }

constructor TQueryText.Create;
begin
  FValueList := TStringList.Create;
end;

destructor TQueryText.Destroy;
begin
  FValueList.Free;
  inherited;
end;

{ TReport ---------------------------------------------------------------------------------- }

{ TReport constructors, destructors, assign, clear }

constructor TReport.Create;
begin
  FQueryList := TList.Create;
  FColList   := TList.Create;
  FParents   := TStringList.Create;
  LoadCriteriaTree(FParents);
  if ReportCache = nil then ReportCache := TReportCache.Create;
end;

destructor TReport.Destroy;
begin
  Clear;
  FQueryList.Free;
  FColList.Free;
  FParents.Free;
  inherited;
end;

procedure TReport.Clear;
var
  i: Integer;
  AQueryText: TQueryText;
begin
  FId := 0;
  FName := '';
  for i := Pred(FQueryList.Count) downto 0 do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    AQueryText.Free;
  end;
  FQueryList.Clear;
  SetColCount(0);
end;

{ TReport private methods }

function TReport.GetColCount: Integer;
{ returns the number of columns in the report }
begin
  Result := FColList.Count;
end;

function TReport.GetColHeader(Index: Integer): string;
{ returns the header text for a specific column }
var
  AColFormat: TColFormat;
begin
  Result := '';
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  Result := AColFormat.Header;
end;

function TReport.GetColName(Index: Integer): string;
{ returns the internal name for the column (maps to 102.24) }
var
  AColFormat: TColFormat;
begin
  Result := '';
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  Result := AColFormat.Name;
end;

function TReport.GetColWidth(Index: Integer): Integer;
{ returns the width in pixels for a specific column }
var
  AColFormat: TColFormat;
begin
  Result := 0;
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  Result := AColFormat.Width
end;

function TReport.FindSequenceBeforeDateRangeQuery: Integer;
var
  i: integer;
  maxSeq,maxSeq1,maxSeq2: integer;
begin
  maxSeq   := 0;
  maxSeq1  := 0;
  maxSeq2  := 0;
  for i := 0 to FQueryList.Count - 1 do
  begin
    if (TQueryText(FQueryList[i]).FCriterionTag = PT_LIST) and (TQueryText(FQueryList[i]).ConstraintName = 'Patient.ListSource') then
    begin
      if maxSeq < 1 then
        maxSeq := TQueryText(FQueryList[i]).FSequence
    end
    else if (TQueryText(FQueryList[i]).FCriterionTag=PT_PRI_PROVIDER) and (TQueryText(FQueryList[i]).ConstraintName = 'Patient.Primary') then
    begin
      if maxSeq1 < 1 then
        maxSeq1 := TQueryText(FQueryList[i]).FSequence;
    end
    else if ((TQueryText(FQueryList[i]).FCriterionTag=PT_ANY_VISIT) or (TQueryText(FQueryList[i]).FCriterionTag=PT_SELECTED_VISIT))
      and (TQueryText(FQueryList[i]).ConstraintName = 'Patient.DateRange') then
    begin
      if maxSeq2 < 1 then
        maxSeq2 := TQueryText(FQueryList[i]).FSequence;
    end;
  end;
  maxSeq := HigherOf(maxSeq,maxSeq1);
  maxSeq := HigherOf(maxSeq,maxSeq2);
  Result := maxSeq;
end;

procedure TReport.SetColCount(AValue: Integer);
{ sets the number of columns in the report -- must be called before other "SetCol" methods }
var
  i: Integer;
  AColFormat: TColFormat;
begin
  for i := Pred(FColList.Count) downto 0 do TColFormat(FColList[i]).Free;
  FColList.Clear;
  if AValue = 0 then Exit;
  for i := 0 to Pred(AValue) do
  begin
    AColFormat := TColFormat.Create;
    FColList.Add(AColFormat);
  end;
end;

procedure TReport.SetColHeader(Index: Integer; AValue: string);
{ sets the header text used for the column (call SetColCount first) }
var
  AColFormat: TColFormat;
begin
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  AColFormat.Header := AValue;
end;

procedure TReport.SetColName(Index: Integer; const Value: string);
{ sets the name for the column -- must match the name in 102.24 }
var
  AColFormat: TColFormat;
begin
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  AColFormat.Name := Value;
end;

procedure TReport.SetColWidth(Index, AValue: Integer);
{ sets the width in pixels for a specific column (call SetColCount first) }
var
  AColFormat: TColFormat;
begin
  if (Index >= FColList.Count) or (Index < 0) then Exit;
  AColFormat := TColFormat(FColList[Index]);
  AColFormat.Width := AValue;
end;
{ TODO -oLori -cHepC : GetQueryText }
function TReport.GetQueryText(Index: Integer): TQueryText;
{ returns a QueryText object (criterion, constraint, caption, editor & values }
begin
  Result := nil;
  if (Index >= 0) and (Index < FQueryList.Count)
    then Result := TQueryText(FQueryList[Index]);
end;

procedure TReport.LoadFormatting(Dest: TList; Src: TStringList);
var
  i: Integer;
  AColFormat: TColFormat;
begin
  AColFormat := nil;
  for i := 0 to Pred(Src.Count) do
  begin
    if Src.Names[i] = 'ColumnName' then
    begin
      AColFormat := TColFormat.Create;
      AColFormat.Name := ValuePart(Src[i]);
      Dest.Add(AColFormat);
    end;
    if AColFormat = nil then Continue;
    if Src.Names[i] = 'ColumnHeader' then AColFormat.Header := ValuePart(Src[i]);
    if Src.Names[i] = 'ColumnWidth'  then AColFormat.Width  := StrToInt(ValuePart(Src[i]));
  end;
end;

procedure TReport.LoadQueryText(Dest: TList; Src: TStringList);
var
  i: Integer;
  AQueryText: TQueryText;
begin
  AQueryText := nil;
  for i := 0 to Pred(Src.Count) do
  begin
    if Src.Names[i] = 'QueryText' then
    begin
      AQueryText := TQueryText.Create;
      AQueryText.FSequence := i + 30;
      Dest.Add(AQueryText);
    end;
    if AQueryText = nil then Continue;
    if Src.Names[i] = 'CriterionTag'   then AQueryText.FCriterionTag   := StrToIntDef(ValuePart(Src[i]), 0);
    if Src.Names[i] = 'Caption'        then AQueryText.FCaption        := ValuePart(Src[i]);
    if Src.Names[i] = 'ConstraintName' then AQueryText.FConstraintName := ValuePart(Src[i]);
    if Src.Names[i] = 'EditorID'       then AQueryText.FEditorID       := StrToIntDef(ValuePart(Src[i]), 0);
    if Src.Names[i] = 'Value'          then AQueryText.Values.Add(ValuePart(Src[i]));
    // adjust date captions so that they are always relative to the current date
    if (AQueryText.FEditorID = EDITOR_DATE) and (AQueryText.Values.Count = 1)
      then AQueryText.FCaption := RelativeDateToCaption(AQueryText.Values[0]);
  end;
end;

procedure TReport.SetDisplayText(const AValue: string);
begin
  FDisplayText := AValue;
end;

procedure TReport.SetID(Value: Integer);
{ retrieves report definition from server and sets up report (query text nodes, etc.) }
var
  ANmValList: TStringList;
begin
  Clear;
  ANmValList := TStringList.Create;
  if Value < 0
    then ReportCache.LoadFromCache(ANmValList, Value)
    else LoadReportDefinition(ANmValList, Value);
  FId          := Value;
  FName        := ANmValList.Values['Name'];
  FDisplayText := ANmValList.Values['DisplayText'];
  if Value = 0 then                           // new custom report
  begin
    FId := ReportCache.GetNextIEN;
    FDisplayText := FDisplayText + ' #' + IntToStr(Abs(FId));
  end;
  LoadFormatting(FColList,  ANmValList);
  LoadQueryText(FQueryList, ANmValList);
  ANmValList.Free;
end;

function SortCriteriaBySeq(Item1, Item2: Pointer): Integer;
{ compare function used by AddCriterion method, can't be an object method }
begin
  Result := 0;
  if TQueryText(Item1).FSequence < TQueryText(Item2).FSequence then Result := -1;
  if TQueryText(Item1).FSequence > TQueryText(Item2).FSequence then Result := 1;
end;

{ TReport public methods }

procedure TReport.AddCriterion(ATag: Integer; FollowPtList: boolean);
{ for building a custom report -- adds the query text nodes of a specific criterion }
var
  i: Integer;
  maxSeq: integer;
  AQuery: TList;
  AQueryText,DTRNGText: TQueryText;
  TextDef: TStringList;
  IsMultiRpt: Integer;
begin
  MaxSeq  := 0;
  IsMultiRpt := 0;
  AQuery := TList.Create;
  TextDef := TStringList.Create;
  LoadReportDefinitionTag(TextDef, ATag);
  LoadQueryText(AQuery, TextDef);

  if FollowPtList then
    MaxSeq := FindSequenceBeforeDateRangeQuery;

  for i := 0 to Pred(AQuery.Count) do
  begin
    AQueryText := TQueryText(AQuery[i]);
    AQueryText.FCriterionTag := ATag;
    //the primary provider and patient visit filter queries should be added before the Seaching Reports query item
    if (FollowPtList) and (MaxSeq>0) then
      AQueryText.FSequence := MaxSeq + i + 1
    else
      AQueryText.FSequence := AQueryText.FCriterionTag + i;
    FQueryList.Add(AQueryText);
  end;

  // this section is used for editting the genecri report
  for i := 0 to FQueryList.Count - 1 do
  begin
    if TQueryText(FQueryList[i]).FCriterionTag = RPT_ORDERS then
      IsMultiRpt := IsMultiRpt + 1
    else if TQueryText(FQueryList[i]).FCriterionTag = RPT_CONSULTS then
      IsMultiRpt := IsMultiRpt + 1
    else if TQueryText(FQueryList[i]).FCriterionTag = RPT_DOCUMENTS then
      IsMultiRpt := IsMultiRpt + 1
    else if TQueryText(FQueryList[i]).FCriterionTag = RPT_VISITS then
      IsMultiRpt := IsMultiRpt + 1
  end;
  if IsMultiRpt > 1 then
  begin
    for i := 0 to FQueryList.Count - 1 do
    begin
      if (TQueryText(FQueryList.Items[i]).FCaption = 'Searching Reports')
        and (TQueryText(FQueryList.Items[i]).CriterionTag = ITEM_DATERANGE) then
         IsMultiRpt := 0;
    end;
  end;
  If IsMultiRpt > 1 then
  begin
    DTRNGText := TQueryText.Create;
    DTRNGText.FCaption      := 'Searching Reports';
    DTRNGText.FCriterionTag := ITEM_DATERANGE;
    DTRNGText.FSequence     := FindSequenceBeforeDateRangeQuery + 1;
    fQueryList.Add(DTRNGText);
    for i := 0 to fQuerylist.Count - 1 do
    begin
      if (TQueryText(FQueryList[i]).FCriterionTag = ITEM_DATERANGE) and
        (TQueryText(FQueryList[i]).FConstraintName = 'Search.DateRange') then
          TQueryText(FQueryList[i]).FSequence := DTRNGText.FSequence + 1;
    end
  end;

  FQueryList.Sort(SortCriteriaBySeq);
  AQuery.Free;
  TextDef.Free;
end;

procedure TReport.RemoveCriterion(ATag: Integer;  UpdateSeq: boolean);
{ cleans up query text nodes when removing a criterion from a report }
var
  i, Parent, Child, MaxSeq, InsIdx: Integer;
  AQueryText, DTRNGText: TQueryText;
  NeedUpdate,NeedAddDtText: boolean;
begin
  MaxSeq := 0;
  for i := Pred(FQueryList.Count) downto 0 do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if AQueryText.FCriterionTag = ATag then
    begin
      AQueryText.Free;
      FQueryList.Delete(i);
    end;
  end;

  // this section was used for editting generic order (check, uncheck report type..)
  NeedAddDtText := True;
  for i := Pred(FQueryList.Count) downto 0 do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if (AQueryText.FCriterionTag = RPT_ORDERS) or
      (AQueryText.FCriterionTag = RPT_CONSULTS) or
      (AQueryText.FCriterionTag = RPT_DOCUMENTS) or
      (AQueryText.FCriterionTag = RPT_VISITS)  then
        NeedAddDtText := False;
  end;
  if NeedAddDtText then
  begin
    for i := 0 to FQueryList.Count - 1 do
    begin
      if (TQueryText(FQueryList.Items[i]).FCaption = 'Searching Reports')
        and (TQueryText(FQueryList.Items[i]).CriterionTag = ITEM_DATERANGE) then
      begin
         NeedAddDtText := False;
         Break;
      end;
    end;
  end;
  if NeedAddDtText then
  begin
    DTRNGText := TQueryText.Create;
    DTRNGText.FCaption      := 'Searching Reports';
    DTRNGText.FCriterionTag := ITEM_DATERANGE;
    DTRNGText.FSequence     := FindSequenceBeforeDateRangeQuery + 1;
    InsIdx := DTRNGText.FSequence;
    FQueryList.Add(DTRNGText);
    for i := 0 to FQueryList.Count - 1 do
    begin
      AQueryText := TQueryText(FQueryList[i]);
      if (AQueryText.FCriterionTag = ITEM_DATERANGE) and (AQueryText.ConstraintName = 'Search.DateRange') then
        TQueryText(FQueryList[i]).FSequence := InsIdx + 1;
    end;
    FQueryList.Sort(SortCriteriaBySeq);
  end;

  // if this Criterion was a parent, recursively remove the children
  for i := 0 to Pred(FParents.Count) do
  begin
    Parent := StrToIntDef(FParents.Names[i], 0);
    if (Parent = 0) or (Parent <> ATag) then Continue;
    Child  := StrToIntDef(ValuePart(FParents[i]), 0);
    RemoveCriterion(Child);
  end;

  if UpdateSeq then
    for i := 0 to FQueryList.Count - 1 do
      if TQueryText(FQueryList[i]).FCriterionTag = 1000500 then
        maxSeq := TQueryText(FQueryList[i]).FSequence;

  NeedUpdate := True;
  if UpdateSeq then
    for i := 0 to FQueryList.Count - 1 do
    begin
      if TQueryText(FQueryList[i]).FCriterionTag = 1011500 then
      begin
        if TQueryText(FQueryList[i]).FSequence = (maxSeq+1) then
          NeedUpdate := False;
        if NeedUpdate then
          Dec(TQueryText(FQueryList[i]).FSequence,1);
      end;
    end;

  NeedUpdate := True;
  if UpdateSeq then
  begin
    for i := 0 to FQueryList.Count - 1 do
    begin
      if (TQueryText(FQueryList[i]).FCriterionTag =1011000) or
        (TQueryText(FQueryList[i]).FCriterionTag = 1010500) then
      begin
        if TQueryText(FQueryList[i]).FSequence = (maxSeq+1) then
          NeedUpdate := False;
        if NeedUpdate then
          Dec(TQueryText(FQueryList[i]).FSequence,1);
      end;
    end;
  end;
end;

function TReport.ContainsConstraint(AName: string): Boolean;
{ returns true if query item (first "." piece of constraint name) is in report definition }
var
  i: Integer;
  AQueryText: TQueryText;
begin
  Result := False;
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if (Pos('.', AName) = 0) and (Piece(AQueryText.FConstraintName, '.', 1) = AName)
      then Result := True;
    if (Pos('.', AName) > 0) and (AQueryText.FConstraintName = AName)
      then Result := True;
    if Result then Break;
  end;
end;

function TReport.ContainsCriterion(ATag: Integer): Boolean;
var
  i: Integer;
  AQueryText: TQueryText;
begin
  Result := False;
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if AQueryText.FCriterionTag = ATag then
    begin
      Result := True;
      Break;
    end;
  end;
end;

procedure TReport.DefinitionToStrings(Dest: TStrings);
var
  i, j: Integer;
  AQueryText: TQueryText;
  AColFormat: TColFormat;
begin
  Dest.Add('IEN=' + IntToStr(Fid));
  Dest.Add('Name=' + FName);
  Dest.Add('DisplayText=' + FDisplayText);
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    Dest.Add('QueryText='       + IntToStr(i+1));
    Dest.Add('CriterionTag='    + IntToStr(AQueryText.CriterionTag));
    Dest.Add('Caption='         + AQueryText.Caption);
    Dest.Add('ConstraintName='  + AQueryText.ConstraintName);
    Dest.Add('EditorID='        + IntToStr(AQueryText.EditorId));
    for j := 0 to Pred(AQueryText.Values.Count) do
    begin
      Dest.Add('Value='         + AQueryText.Values[j]);
    end; {for j}
  end; {for i}
  for i := 0 to Pred(FColList.Count) do
  begin
    AColFormat := TColFormat(FColList[i]);
    Dest.Add('Column='       + IntToStr(i));
    Dest.Add('ColumnName='   + AColFormat.Name);
    Dest.Add('ColumnHeader=' + AColFormat.Header);
    Dest.Add('ColumnWidth='  + IntToStr(AColFormat.Width));
  end;
end;

procedure TReport.QueryToStrings(Dest: TStrings);
var
  i, j: Integer;
  AQueryText: TQueryText;
  AColFormat: TColFormat;
begin
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if AQueryText.ConstraintName = '' then Continue;
    for j := 0 to Pred(AQueryText.Values.Count) do
    begin
      Dest.Add(AQueryText.ConstraintName + '=' + AQueryText.Values[j]);
    end;
  end;
  for i := 0 to Pred(FColList.Count) do
  begin
    AColFormat := TColFormat(FColList[i]);
    Dest.Add('Report.Column=' + AColFormat.Name);
  end;
end;

procedure TReport.Delete;
begin
  if not IsOwner then Exit;
  if FId > 0 then DeleteReport(FId) else ReportCache.RemoveFromCache(FId);
  Clear;
end;

function TReport.IsOwner: Boolean;
begin
  if FId > 0 then Result := ReportOwnedByUser(FId) else Result := True;
end;

function TReport.ExistsOnServer: Boolean;
begin
  Result := GetReportIdForUser(FName) <> 0;
end;

procedure TReport.ListNamesByUser(Dest: TStrings);
{ populates a list (i.e., listbox) with the reports that are available (ien^display name) }
var
  tmpList: TStringList;
begin
  tmpList := TStringList.Create;
  LoadReportList(tmpList);
  Dest.Assign(tmpList);
  tmpList.Clear;
  ReportCache.FillCachedReportList(tmpList);
  Dest.AddStrings(tmpList);
  tmpList.Free;
end;

procedure TReport.FillNameValueList(Dest: TStrings);
{ builds a list of name=value pairs to pass to server to execute a query }
var
  i, j: Integer;
  AQueryText: TQueryText;
begin
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    for j := 0 to Pred(AQueryText.FValueList.Count) do
      Dest.Add(AQueryText.FConstraintName + '=' + AQueryText.FValueList[j]);
  end;
end;

procedure TReport.ValuesByConstraint(AValueList: TStrings; const AConstraint: string);
var
  i: Integer;
  AQueryText: TQueryText;
begin
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if AQueryText.FConstraintName = AConstraint then
    begin
      AValueList.Assign(AQueryText.FValueList);
      Break;
    end; {if AQueryText}
  end; {for i}
end;

procedure TReport.Save(ToServer: Boolean);
{ Saves the report either to cache or the server }
var
  NewIEN: Integer;
  SaveStatus: string;
  ANmValList: TStringList;
begin
  ANmValList := TStringList.Create;
  DefinitionToStrings(ANmValList);
  if ToServer then
  begin
    SaveStatus := SaveReportDefinition(ANmValList);
    NewIEN := StrToIntDef(Piece(SaveStatus, U, 1), 0);
    if NewIEN > 0 then
    begin
      ReportCache.RemoveFromCache(FId);
      FId := NewIEN;
    end
    else Clear;
  end
  else ReportCache.SaveToCache(ANmValList);
  ANmValList.Free;
end;

function TReport.ValuesAreMissing: Boolean;
{ returns true if any constraint values are missing (would prevent report from running) }
var
  i: Integer;
  AQueryText: TQueryText;
begin
  Result := False;
  for i := 0 to Pred(FQueryList.Count) do
  begin
    AQueryText := TQueryText(FQueryList[i]);
    if (AQueryText.FEditorId > 0) and (AQueryText.FEditorId <> EDITOR_NONE) then
    begin
      if AQueryText.FValueList.Count = 0 then Result := True;
    end;
  end;
end;

{ TReportCache }

constructor TReportCache.Create;
begin
  FNextIEN := -1;
  FCache := TList.Create;
end;

destructor TReportCache.Destroy;
var
  i: Integer;
begin
  for i := Pred(FCache.Count) downto 0 do TStringList(FCache[i]).Free;
  FCache.Free;
  inherited;
end;

procedure TReportCache.FillCachedReportList(Dest: TStrings);
var
  i: Integer;
  ATextDef: TStringList;
begin
  for i := 0 to Pred(FCache.Count) do
  begin
    ATextDef := TStringList(FCache[i]);
    Dest.Add(ATextDef.Values['IEN'] + U + ATextDef.Values['DisplayText']);
  end;
end;

function TReportCache.GetNextIEN: Integer;
begin
  Result := FNextIEN;
end;

procedure TReportCache.LoadFromCache(ATextDef: TStringList; AnIEN: Integer);
var
  i: Integer;
  CachedTextDef: TStringList;
begin
  for i := 0 to Pred(FCache.Count) do
  begin
    CachedTextDef := TStringList(FCache[i]);
    if StrToIntDef(CachedTextDef.Values['IEN'], 0) = AnIEN then
    begin
      ATextDef.Assign(CachedTextDef);
      break;
    end; {if StrToIntDef}
  end; {for i}
end;

procedure TReportCache.RemoveFromCache(AnIEN: Integer);
var
  i, CachedIndex: Integer;
  CachedTextDef: TStringList;
begin
  CachedIndex := -1;
  for i := 0 to Pred(FCache.Count) do
  begin
    CachedTextDef := TStringList(FCache[i]);
    if StrToIntDef(CachedTextDef.Values['IEN'], 0) = AnIEN then CachedIndex := i;
  end; {for i}
  if CachedIndex > -1 then
  begin
    TStringList(FCache[CachedIndex]).Free;
    FCache.Delete(CachedIndex);
  end;
end;

procedure TReportCache.SaveToCache(ATextDef: TStringList);
var
  i, AnIEN, CachedIndex: Integer;
  CachedTextDef: TStringList;
begin
  AnIEN := StrToIntDef(ATextDef.Values['IEN'], 0);
  CachedIndex := -1;
  for i := 0 to Pred(FCache.Count) do
  begin
    CachedTextDef := TStringList(FCache[i]);
    if StrToIntDef(CachedTextDef.Values['IEN'], 0) = AnIEN then CachedIndex := i;
  end; {for i}
  if CachedIndex < 0 then
  begin
    CachedTextDef := TStringList.Create;
    CachedTextDef.Assign(ATextDef);
    FCache.Add(CachedTextDef);
    Dec(FNextIEN);
  end else
  begin
    CachedTextDef := TStringList(FCache[CachedIndex]);
    CachedTextDef.Assign(ATextDef);
  end;
end;

procedure SaveEditedReport;
const
  TX_SAVE = 'Do you want to save the changes you made to the ';
  TX_NO_VALUES = 'The report cannot be built until values for all criteria are set.' + CRLF +
                 'Click the blue links in the query to add necessary values.';
  TC_NO_VALUES = 'Missing Values';
  TX_NO_REPORT = 'A report must first be selected or created.';
  TC_NO_REPORT = 'No Report Selected';
var
  CurQuery: TStringList;
begin
  CurQuery := TStringList.Create;
  Report.QueryToStrings(CurQuery);
  if (OldQuery.Count > 0) and (not OldQuery.Equals(CurQuery)) then
  begin
    if MessageDlg(TX_SAVE + Report.DisplayText + '?', mtWarning,[mbYes,mbNo],0) = mrYes then
       Report.Save(True)
    else
       Report.Save(False);
  end else
    Report.Save(True);
  if ShowMsgOn(Report.Id = 0,           TX_NO_REPORT, TC_NO_REPORT) then Exit;
  CurQuery.Free;
  OldQuery.Clear;
end;

function IsGenericReport(ARPTDisplayName: string): boolean;
var
  i: integer;
  GenericRPTList: TStringList;
begin
  Result := False;
  GenericRPTList := TStringList.Create;
  GenericReportList(GenericRPTList);
  for i := 0 to GenericRPTList.Count - 1 do
  begin
    if AnsiCompareText(ARPTDisplayName,Piece(GenericRPTList[i],'^',2))=0 then
      Result := True;
  end;
end;

end.
