unit fDateRange;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Mask, uReport, DateUtils, ORFn;

type
  TfrmDateRange = class(TForm)
    btnOK: TButton;
    btnCancel: TButton;
    pnlByDates: TPanel;
    lblFrom: TLabel;
    lblThru: TLabel;
    calThru: TDateTimePicker;
    chkNoStart: TCheckBox;
    chkNoStop: TCheckBox;
    pnlByYear: TPanel;
    cboQuarter: TComboBox;
    chkQuarter: TCheckBox;
    txtYear: TMaskEdit;
    spnYear: TUpDown;
    grpRange: TGroupBox;
    radByDates: TRadioButton;
    radByYear: TRadioButton;
    radYearCalendar: TRadioButton;
    radYearFiscal: TRadioButton;
    radToday: TRadioButton;
    radPast24: TRadioButton;
    radPastWeek: TRadioButton;
    radPastMonth: TRadioButton;
    radThisQuarter: TRadioButton;
    radThisYear: TRadioButton;
    calFrom: TDateTimePicker;
    procedure radByDatesClick(Sender: TObject);
    procedure radByYearClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure chkQuarterClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure chkNoStartClick(Sender: TObject);
    procedure chkNoStopClick(Sender: TObject);
    procedure radPresetClick(Sender: TObject);
    procedure radYearFiscalClick(Sender: TObject);
    procedure radYearCalendarClick(Sender: TObject);
    procedure calFromExit(Sender: TObject);
    procedure txtYearExit(Sender: TObject);
    procedure txtYearChange(Sender: TObject);
  private
    FCurrentMonth:  Integer;
    FCurrentYear:   Integer;
    FCurrentFiscal: Integer;
    FAbnormalDate:  TFMDateTime;
    FCheckAbnormalDate: boolean;
    FRelativeValue: string;
    function GetCaptionText: string;
    function GetRelativeValue: string;
    function GetRelativeDates: string;
    function GetRelativeYear: string;
    procedure SetupContent(x: string);
    procedure SetCalendarYear(AYear: Integer);
    procedure SetFiscalYear(AYear: Integer);
    procedure SetContentForDates(const x: string);
    procedure SetContentForYear(const x: string);
    function checkAbnormalStartDate(DateRange: string): boolean;
 public
    class procedure InvokeEditor(var ACaption: string; ValueList: TStringList; NeedAbnormalDate: boolean = False);
  end;

var
  frmDateRange: TfrmDateRange;

implementation

{$R *.DFM}

uses  rDateTime,rReport;

class procedure TfrmDateRange.InvokeEditor(var ACaption: string; ValueList: TStringList; NeedAbnormalDate: boolean = False);
var
  frmDateRange: TfrmDateRange;
begin
  frmDateRange := TfrmDateRange.Create(Application);
  try
    if ValueList.Count = 1 then frmDateRange.SetupContent(ValueList[0]);
    if NeedAbnormalDate then
    begin
       frmDateRange.FCheckAbnormalDate := NeedAbnormalDate;
       frmDateRange.FAbnormalDate := MakeFMDateTime(StartDateForAbnormal);
    end;
    if frmDateRange.ShowModal = mrOK then
    begin
      ValueList.Clear;
      if NeedAbnormalDate then
      begin
        if Length(frmDateRange.FRelativeValue)>0 then
          ValueList.Add(frmDateRange.FRelativeValue)
        else
          ValueList.Add(frmDateRange.GetRelativeValue);
      end
      else
        ValueList.Add(frmDateRange.GetRelativeValue);
      ACaption := frmDateRange.GetCaptionText;
    end;
  finally
    frmDateRange.Release;
  end;
end;

procedure TfrmDateRange.FormCreate(Sender: TObject);
begin
  FCurrentMonth := (Trunc(FMToday) mod 10000) div 100;   // extract month from Fileman date
  FCurrentYear  := Trunc(FMToday / 10000) + 1700;        // extract year from Fileman date
  if FCurrentMonth > 9
    then FCurrentFiscal := FCurrentYear + 1
    else FCurrentFiscal := FCurrentYear;
  calFrom.DateTime := Now;
  calThru.DateTime := Now;
  FAbnormalDate := FMNow;
  FCheckAbnormalDate := False;
  FRelativeValue := '';
  radByDates.Checked := True;
  radByDatesClick(Self);
end;

function TfrmDateRange.GetCaptionText: string;
var
  YearType: string;
begin
  Result := '< select date range >';
  if radToday.Checked       then Result := 'during Today';
  if radPast24.Checked      then Result := 'during the Past 24 Hours';
  if radPastWeek.Checked    then Result := 'during the Past Week';
  if radPastMonth.Checked   then Result := 'during the Past Month';
  if radThisQuarter.Checked then Result := 'during the Current Quarter';
  if radThisYear.Checked    then Result := 'during the Current Year';
  if pnlByYear.Visible then
  begin
    Result := 'during ';
    if chkQuarter.Checked then Result := Result + cboQuarter.Items[cboQuarter.ItemIndex] + ' ';
    if radYearFiscal.Checked then YearType := ' FY' else YearType := '';
    Result := Result + YearType + txtYear.Text;
  end;
  if pnlByDates.Visible then
  begin
    Result := '';
    if not chkNoStart.Checked
      then Result := Result + 'from ' + FormatDateTime('dd mmm yyyy', calFrom.Date);
    if not chkNoStop.Checked
      then Result := Result + ' through ' + FormatDateTime('dd mmm yyyy', calThru.Date);
    if chkNoStart.Checked and chkNoStop.Checked then Result := 'any date';
  end;
end;

function TfrmDateRange.GetRelativeYear: string;
var
  ThisYear, TargetYear, Offset: Integer;
begin
  if radYearFiscal.Checked
    then Result := 'YF'
    else Result := 'YC';
  TargetYear := spnYear.Position;
  if radYearFiscal.Checked and (FCurrentMonth > 9)
     then ThisYear := FCurrentYear + 1
     else ThisYear := FCurrentYear;
  Offset := TargetYear - ThisYear;
  if Offset > 0 then Result := Result + '+' + IntToStr(Offset);
  if Offset < 0 then Result := Result + IntToStr(Offset);
  if chkQuarter.Checked then Result := Result + 'Q' + IntToStr(cboQuarter.ItemIndex + 1);
end;

function TfrmDateRange.GetRelativeDates: string;
var
  FMStart, FMStop: TFMDateTime;
begin
  Result := '';
  if chkNoStart.Checked then Result := '0'
  else
  begin
    FMStart := DateTimeToFMDateTime(calFrom.Date);
    Result := FloatToStr(FMStart);
    Result := Piece(Result,'.',1);
  end;
  Result := Result + ':';
  if chkNoStop.Checked then Result := Result + '0'
  else
  begin
    FMStop  := DateTimeToFMDateTIme(calThru.Date);
    Result := Result + Piece(FloatToStr(FMStop),'.',1);
  end;
  if Result = ':' then Result := '';
end;

function TfrmDateRange.GetRelativeValue: string;
begin
  Result := '';
  if radToday.Checked       then Result := 'T:T';
  if radPast24.Checked      then Result := 'T-1:T';
  if radPastWeek.Checked    then Result := 'T-7:T';
  if radPastMonth.Checked   then Result := 'T-30:T';
  if radThisQuarter.Checked then Result := 'YFQ';
  if radThisYear.Checked    then Result := 'YF';
  if radByDates.Checked     then Result := GetRelativeDates;
  if radByYear.Checked      then Result := GetRelativeYear;
end;

procedure TfrmDateRange.SetCalendarYear(AYear: Integer);
const
  TX_CY_FIRST  = '1st Quarter (Jan-Mar)';
  TX_CY_SECOND = '2nd Quarter (Apr-Jun)';
  TX_CY_THIRD  = '3rd Quarter (Jul-Sep)';
  TX_CY_FOURTH = '4th Quarter (Oct-Dec)';
begin
  radYearCalendar.Checked := True;
  cboQuarter.Clear;
  cboQuarter.Items.Add(TX_CY_FIRST);
  cboQuarter.Items.Add(TX_CY_SECOND);
  cboQuarter.Items.Add(TX_CY_THIRD);
  cboQuarter.Items.Add(TX_CY_FOURTH);
  case FCurrentMonth of
    1..3: cboQuarter.ItemIndex := 0;
    4..6: cboQuarter.ItemIndex := 1;
    7..9: cboQuarter.ItemIndex := 2;
  10..12: cboQuarter.ItemIndex := 3;
  end;
  spnYear.Position := AYear;
end;

procedure TfrmDateRange.SetFiscalYear(AYear: Integer);
const
  TX_FY_FIRST  = '1st Quarter (Oct-Dec)';
  TX_FY_SECOND = '2nd Quarter (Jan-Mar)';
  TX_FY_THIRD  = '3rd Quarter (Apr-Jun)';
  TX_FY_FOURTH = '4th Quarter (Jul-Sep)';
begin
  radYearFiscal.Checked := True;
  cboQuarter.Clear;
  cboQuarter.Items.Add(TX_FY_FIRST);
  cboQuarter.Items.Add(TX_FY_SECOND);
  cboQuarter.Items.Add(TX_FY_THIRD);
  cboQuarter.Items.Add(TX_FY_FOURTH);
  case FCurrentMonth of
    1..3: cboQuarter.ItemIndex := 3;
    4..6: cboQuarter.ItemIndex := 0;
    7..9: cboQuarter.ItemIndex := 1;
  10..12: cboQuarter.ItemIndex := 2;
  end;
  spnYear.Position := AYear;
end;

procedure TfrmDateRange.SetContentForYear(const x: string);
var
  Offset, Qtr: Integer;
begin
  radByYear.Checked := True;
  radByYearClick(Self);
  Offset := 0;
  if Pos('+', x) > 0 then Offset :=   StrToIntDef(Piece(Piece(x, '+', 2), 'Q', 1), 0);
  if Pos('-', x) > 0 then Offset := -(StrToIntDef(Piece(Piece(x, '-', 2), 'Q', 1), 0));
  if x[2] = 'C' then SetCalendarYear(FCurrentYear + Offset);
  if x[2] = 'F' then SetFiscalYear(FCurrentFiscal + Offset);
  Qtr := StrToIntDef(Piece(x, 'Q', 2), 0);
  if Qtr = 0 then
  begin
    chkQuarter.Checked := False;
    cboQuarter.Enabled := False;
  end else
  begin
    chkQuarter.Checked := True;
    cboQuarter.Enabled := True;
    cboQuarter.ItemIndex := Qtr - 1;
  end;
end;

procedure TfrmDateRange.SetContentForDates(const x: string);
var
  FMStart, FMStop: TFMDateTime;
begin
  chkNoStart.Checked := (Piece(x, ':', 1) = '') or (Piece(x, ':', 1) = '0');
  chkNoStop.Checked  := (Piece(x, ':', 2) = '') or (Piece(x, ':', 2) = '0');
  chkNoStartClick(Self);
  chkNoStopClick(Self);
  FMStart := StrToFMDateTime(Piece(x, ':', 1));
  FMStop  := StrToFMDateTime(Piece(x, ':', 2));
  if not chkNoStart.Checked then calFrom.Date := FMDateTimeToDateTime(FMStart);
  if not chkNoStop.Checked  then calThru.Date := FMDateTimeToDateTime(FMStop);
end;

procedure TfrmDateRange.SetupContent(x: string);
begin
  if      x = 'T:T'    then radToday.Checked       := True
  else if x = 'T-1:T'  then radPast24.Checked      := True
  else if x = 'T-7:T'  then radPastWeek.Checked    := True
  else if x = 'T-30:T' then radPastMonth.Checked   := True
  else if x = 'YFQ'    then radThisQuarter.Checked := True
  else if x = 'YF'     then radThisYear.Checked    := True
  else if CharAt(x, 1) = 'Y' then SetContentForYear(x)
  else SetContentForDates(x);
end;

procedure TfrmDateRange.chkNoStartClick(Sender: TObject);
begin
  calFrom.Enabled := not chkNoStart.Checked;
end;

procedure TfrmDateRange.chkNoStopClick(Sender: TObject);
begin
  calThru.Enabled := not chkNoStop.Checked;
end;

procedure TfrmDateRange.radPresetClick(Sender: TObject);
var
  DTrange: string;
begin
  pnlByDates.Visible := False;
  pnlByYear.Visible := False;
  if radToday.Checked       then DTrange := 'T:T';
  if radPast24.Checked      then DTrange := 'T-1:T';
  if radPastWeek.Checked    then DTrange := 'T-7:T';
  if radPastMonth.Checked   then DTrange := 'T-30:T';
  if radThisQuarter.Checked then DTrange := 'YFQ';
  if radThisYear.Checked    then DTrange := 'YF';
  CheckAbnormalStartDate(DTrange);
end;

procedure TfrmDateRange.radByDatesClick(Sender: TObject);
begin
  pnlByDates.Visible := True;
  pnlByYear.Visible := False;
end;

procedure TfrmDateRange.radByYearClick(Sender: TObject);
begin
  SetCalendarYear(FCurrentYear);  // always default to current calendar year?
  pnlByYear.Visible := True;
  pnlByDates.Visible := False;
end;

procedure TfrmDateRange.radYearFiscalClick(Sender: TObject);
var
  AQtr, AYear: Integer;
begin
  AQtr  := cboQuarter.ItemIndex;
  AYear := spnYear.Position;                              // calendar to fiscal
  if radYearFiscal.Checked then
  begin
    if AQtr = 3 then AQtr := 0 else Inc(AQtr);
    if chkQuarter.Checked and (AQtr = 0) then Inc(AYear);
    SetFiscalYear(AYear);
    cboQuarter.ItemIndex := AQtr;
  end;
end;

procedure TfrmDateRange.radYearCalendarClick(Sender: TObject);
var
  AQtr, AYear: Integer;
begin
  AQtr  := cboQuarter.ItemIndex;
  AYear := spnYear.Position;                              // fiscal to calendar
  if radYearCalendar.Checked then
  begin
    if AQtr = 0 then AQtr := 3 else Dec(AQtr);
    if chkQuarter.Checked and (AQtr = 3) then Dec(AYear);
    SetCalendarYear(AYear);
    cboQuarter.ItemIndex := AQtr;
  end;
end;

procedure TfrmDateRange.chkQuarterClick(Sender: TObject);
begin
  cboQuarter.Enabled := chkQuarter.Checked;
end;

procedure TfrmDateRange.btnCancelClick(Sender: TObject);
begin
  Self.ModalResult := mrCancel;
end;

procedure TfrmDateRange.btnOKClick(Sender: TObject);
var
  DTrange: string;
begin
  if radByYear.Checked then
  begin
    DTrange := GetRelativeYear;
    if checkAbnormalStartDate(DTrange) then
      Exit
  end;
  Self.ModalResult := mrOK;
end;

procedure TfrmDateRange.calFromExit(Sender: TObject);
const
  TXT4 = 'The start date can not be greater than stop date.';
var
  DTrange: string;
begin
  if calThru.Enabled and (CompareDate(calThru.DateTime,calFrom.DateTime)<0) then
  begin
    ShowMessage(TXT4);
    calFrom.DateTime := Now;
    Exit;
  end;
  DTrange := GetRelativeDates;
  checkAbnormalStartDate(DTrange);
end;

function TfrmDateRange.checkAbnormalStartDate(DateRange: string): boolean;
const
  TXT1 = 'The earliest available abnormal report date for your facility is ';
  TXT2 = #13 + 'You can not search for abnormal results prior to that date.';
var
  TheDateRng: string;
  TheStartDate,TheEndDate: TFMDateTime;
begin
  Result := False;
  if FCheckAbnormalDate then
  begin
    TheDateRng := '';
    TheDateRng := FMDateRange(DateRange);
    if Length(TheDateRng)>0 then
    begin
      TheStartDate := MakeFMDateTime(Piece(TheDateRng,':',1));
      TheEndDate   := MakeFMDateTime(Piece(TheDateRng,':',2));
      if CompareDate(FAbnormalDate,TheStartDate) > 0 then
      begin
        MessageDlg(TXT1 + DateToStr(FMDateTimeToDateTime(FAbnormalDate)) + TXT2, mtWarning, [mbOK],0);
        FRelativeValue := MakeRelativeDateTime(FAbnormalDate) + ':' + MakeRelativeDateTime(TheEndDate);
        radByDates.Checked := True;
        ActiveControl := radByDates;
        calFrom.DateTime := FMDateTimeToDateTime(FAbnormalDate);
        if CompareDate(FAbnormalDate, TheEndDate) > 0 then
          TheEndDate := FMNow;
        calThru.DateTime := FMDateTimeToDateTime(TheEndDate);
        Result := True;
      end;
    end;
  end;
end;

procedure TfrmDateRange.txtYearExit(Sender: TObject);
var
  DTrange: string;
begin
  DTrange := GetRelativeYear;
  checkAbnormalStartDate(DTrange);
end;

procedure TfrmDateRange.txtYearChange(Sender: TObject);
var
  theY,i: integer;
  tempStr: string;
begin
  tempStr := txtYear.Text;
  while Length(tempStr) < 4 do
    tempStr := tempStr + '0';
  for i := 1 to Length(tempStr) do
  begin
    if tempStr[i] = ' ' then
      tempStr[i] := '0';
  end;
  txtYear.Text := tempStr;
  theY := StrToInt(txtYear.Text);
  if theY <= 25 then
  begin
    theY := 2000 + theY;
    txtYear.Text := IntToStr(theY);
  end else if (TheY > 25) and (TheY < 100) then
  begin
    theY := 1900 + theY;
    txtYear.Text := IntToStr(theY);
  end else if (TheY>=100) and (TheY < 1000) then
  begin
    theY := 1000 + theY;
    txtYear.Text := IntToStr(TheY);
  end;
end;

end.
