unit fHost;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Tabs, ComCtrls,
  ExtCtrls, Menus, StdCtrls; 

const
  CONTENT_REPORTS   =  1;
  CONTENT_PATIENTS  =  2;
  CONTENT_CRITERIA  =  3;
  CONTENT_ORDERS    =  4;
  CONTENT_CONSULTS  =  5;
  CONTENT_DOCUMENTS =  6;
  CONTENT_VISITS    =  7;
  CONTENT_FORMAT    =  8;
  CONTENT_SAVERPT   =  9;
  CONTENT_DISPLAY   = 10;

type
  TfrmHost = class(TForm)
    pnlBody: TPanel;
    mnHost: TMainMenu;
    Report1: TMenuItem;
    Bevel1: TBevel;
    mnHelp: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private
    FCurrentForm: TForm;
    FWizardPos: Integer;
    FWizardShow: array[CONTENT_CRITERIA..CONTENT_SAVERPT] of Boolean;
    FDriveByEdit: Boolean;
    function GetWizardShow(Index: Integer): Boolean;
    procedure SetWizardShow(Index: Integer; const Value: Boolean);
  public
    procedure SwitchTo(ATargetForm: Integer);
    procedure WizardBack;
    procedure WizardNext;
    property WizardShow[Index: Integer]: Boolean read GetWizardShow write SetWizardShow;
    property DriveByEdit: Boolean read FDriveByEdit write FDriveByEdit;
  end;

var
  frmHost: TfrmHost;

implementation

{$R *.DFM}

uses ORNet, ORFn, uReport, uCohort, fPageReports, fPageDisplay, fPageSave, fPageFormat,
     fPageCriteria, fPageOrders, fPageConsults, fPageDocuments, fPageVisits, rReport;

const
  TX_WARN = 'The Reports generated by this query tool will contain Confidential Information.' +
            CRLF + CRLF + 'Do you wish to continue?';
  TC_WARN = 'Confidential Patient Information';

{ Create and Destroy ----------------------------------------------------------------------- }

procedure TfrmHost.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  FDriveByEdit := False;
  if ConnectToServer('ORRCM REPORTING') then
  begin
    Report  := TReport.Create;
    OldQuery := TStringList.Create;
    Cohort  := TCohort.Create;
    SwitchTo(CONTENT_REPORTS);
  end
  else Application.Terminate;
  for i := Low(FWizardShow) to High(FWizardShow) do FWizardShow[i] := True;
end;

procedure TfrmHost.FormShow(Sender: TObject);
begin
  if InfoBox(TX_WARN, TC_WARN, MB_YESNO) = IDNO
    then Application.Terminate;
end;

procedure TfrmHost.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ClearQuery;  // to clean up temp globals
end;

procedure TfrmHost.FormDestroy(Sender: TObject);
begin
  Report.Free;
  Cohort.Free;
  OldQuery.Free;
end;

{ Public Methods --------------------------------------------------------------------------- }

procedure TfrmHost.SwitchTo(ATargetForm: Integer);
type
  TFormClass = class of TForm;
var
  AFormClass: TFormClass;
begin
  if FCurrentForm <> nil then FCurrentForm.Release;
  case ATargetForm of
  CONTENT_REPORTS   : AFormClass := TfrmPageReports;
  CONTENT_CRITERIA  : AFormClass := TfrmPageCriteria;
  CONTENT_ORDERS    : AFormClass := TfrmPageOrders;
  CONTENT_CONSULTS  : AFormClass := TfrmPageConsults;
  CONTENT_DOCUMENTS : AFormClass := TfrmPageDocuments;
  CONTENT_VISITS    : AFormClass := TfrmPageVisits;
  CONTENT_FORMAT    : AFormClass := TfrmPageFormat;
  CONTENT_SAVERPT   : AFormClass := TfrmPageSave;
  CONTENT_DISPLAY   : AFormClass := TfrmPageDisplay;
  else                AFormClass := nil;
  end;
  FCurrentForm := AFormClass.Create(Application);
  FCurrentForm.Parent  := Self.pnlBody;
  FCurrentForm.Align   := alClient;
  FCurrentForm.Visible := True;
  FWizardPos := ATargetForm;
  case ATargetForm of
  CONTENT_CRITERIA  : TfrmPageCriteria(FCurrentForm).chkScreenVisit.SetFocus;
  CONTENT_ORDERS    : TfrmPageOrders(FCurrentForm).chkTypeOrder.SetFocus;
  CONTENT_CONSULTS  : TfrmPageConsults(FCurrentForm).chkTypeConsult.SetFocus;
  CONTENT_DOCUMENTS : TfrmPageDocuments(FCurrentForm).chkTypeClass.SetFocus;
  CONTENT_VISITS    : TfrmPageVisits(FCurrentForm).chkLocation.SetFocus;
  CONTENT_FORMAT    : TfrmPageFormat(FCurrentForm).lstAvailable.SetFocus;
  CONTENT_SAVERPT   : TfrmPageSave(FCurrentForm).txtRptName.SetFocus;
  CONTENT_DISPLAY   : TfrmPageDisplay(FCurrentForm).btnPrint.SetFocus;
  end;
  if ATargetForm =   CONTENT_REPORTS then
    mnHost.Merge(TfrmPageReports(FCurrentForm).mnuReport)
  else
    mnHost.Items.Clear;
end;

function TfrmHost.GetWizardShow(Index: Integer): Boolean;
begin
  Result := FWizardShow[Index];
end;

procedure TfrmHost.SetWizardShow(Index: Integer; const Value: Boolean);
begin
  FWizardShow[Index] := Value;
end;

procedure TfrmHost.WizardBack;
var
  i: Integer;
begin
  for i := Pred(FWizardPos) downto CONTENT_CRITERIA do
  begin
    if FWizardShow[i] then
    begin
      FWizardPos := i;
      SwitchTo(i);
      Break;
    end;
  end;
end;

procedure TfrmHost.WizardNext;
var
  i: Integer;
begin
  for i := Succ(FWizardPos) to CONTENT_SAVERPT do
  begin
    if FWizardShow[i] then
    begin
      FWizardPos := i;
      SwitchTo(i);
      Break;
    end;
  end;
end;


end.
