unit fMultiCheck;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, CheckLst, ExtCtrls, ORCtrls;

const
  STS_ORDER      = 1;
  STS_ORDER_SIGN = 2;
  STS_DOCUMENT   = 3;
  STS_CLASS      = 4;

type
  TfrmMultiCheck = class(TForm)
    lblInstruct: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    lstChecks: TORListBox;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure lstChecksClickCheck(Sender: TObject; Index: Integer);
  private
    procedure GetCheckStates(var ACaption: string; DestList: TStringList);
    procedure SetCheckStates(SrcList: TStringList);
    procedure SetupContent(StatusType: Integer);
  public
    class procedure InvokeEditor(var ACaption: string; ValueList: TStringList;
      const StatusType: Integer);
  end;

implementation

{$R *.DFM}

uses ORFn, rReport;

const
  TX_STS_ORDER      = 'Order Status';
  TX_STS_ORDER_SIGN = 'Order Signature Status';
  TX_STS_DOCUMENT   = 'Document Status';
  TX_STS_CLASS      = 'Document Class';

{ TfrmMultiCheck }

class procedure TfrmMultiCheck.InvokeEditor(var ACaption: string; ValueList: TStringList;
  const StatusType: Integer);
var
  frmMultiCheck: TfrmMultiCheck;
begin
  frmMultiCheck := TfrmMultiCheck.Create(Application);
  try
    ResizeAnchoredFormToFont(frmMultiCheck);
    frmMultiCheck.SetupContent(StatusType);
    frmMultiCheck.SetCheckStates(ValueList);
    if frmMultiCheck.ShowModal = mrOK then
    begin
      frmMultiCheck.GetCheckStates(ACaption, ValueList);
    end; {if ShowModal}
  finally
    frmMultiCheck.Release;
  end;
end;

procedure TfrmMultiCheck.SetupContent(StatusType: Integer);
begin
  case StatusType of
    STS_ORDER:
    begin
      LoadOrderStatuses(lstChecks.Items);
      Caption := TX_STS_ORDER;
    end;
    STS_ORDER_SIGN:
    begin
      LoadOrderSignStatuses(lstChecks.Items);
      Caption := TX_STS_ORDER_SIGN;
    end;
    STS_DOCUMENT:
    begin
      LoadDocumentStatuses(lstChecks.Items);
      Caption := TX_STS_DOCUMENT;
    end;
    STS_CLASS:
    begin
      LoadDocumentClasses(lstChecks.Items);
      Caption := TX_STS_CLASS;
    end;
  end; {case}
end;

procedure TfrmMultiCheck.GetCheckStates(var ACaption: string; DestList: TStringList);
var
  i: Integer;
  ADelim: string;
begin
  ACaption := '';
  ADelim := '';
  DestList.Clear;
  for i := 0 to Pred(lstChecks.Items.Count) do
  begin
    if lstChecks.Checked[i] then
    begin
      DestList.Add(Piece(lstChecks.Items[i], '^', 1));
      if DestList.Count > 1 then ADelim := ' or ';
      ACaption := ACaption + ADelim + Piece(lstChecks.Items[i], '^', 2);
    end; {if lstChecks}
  end; {for j}
end;

procedure TfrmMultiCheck.SetCheckStates(SrcList: TStringList);
var
  i, j: Integer;
begin
  for i := 0 to Pred(SrcList.Count) do
  begin
    for j := 0 to Pred(lstChecks.Items.Count) do
      if SrcList[i] = Piece(lstChecks.Items[j], '^', 1) then lstChecks.Checked[j] := True;
  end; {for i}
end;

procedure TfrmMultiCheck.btnOKClick(Sender: TObject);
begin
  Self.ModalResult := mrOK;
end;

procedure TfrmMultiCheck.btnCancelClick(Sender: TObject);
begin
  Self.ModalResult := mrCancel;
end;

procedure TfrmMultiCheck.lstChecksClickCheck(Sender: TObject; Index: Integer);
var
  i: Integer;
  FoundOne: Boolean;
begin
  FoundOne := False;
  for i := 0 to Pred(lstChecks.Items.Count) do
  begin
    if lstChecks.Checked[i] then FoundOne := True;
    if FoundOne then Break;
  end;
  btnOK.Enabled := FoundOne;
end;

end.
