unit fMultiList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ORCtrls, ExtCtrls, Menus, Buttons, ComCtrls;

const
  // types of selection items
  ITM_PROVIDER  = 1;                     // Provider file
  ITM_CLINIC    = 2;                     // Hospital Location file
  ITM_DOCUMENT  = 3;                     // Document Definition file
  ITM_ORDERABLE = 4;                     // Orderable Item file
  ITM_SERVICE   = 5;                     // Display Group file
  ITM_CONSULT   = 6;                     // subset of Display Group file
  ITM_PROCEDURE = 7;                     // subset of Orderable Item file

type
  TfrmMultiList = class(TForm)
    cboAvailable: TORComboBox;
    btnOK: TButton;
    btnCancel: TButton;
    btnAdd: TButton;
    lblSelected: TLabel;
    popList: TPopupMenu;
    popListDelete: TMenuItem;
    popListProperties: TMenuItem;
    N2: TMenuItem;
    lblAvailable: TLabel;
    btnRemove: TButton;
    Label1: TLabel;
    Label2: TLabel;
    lblFilterInstruct: TLabel;
    lblDblClick: TLabel;
    lblFilter: TLabel;
    cboFilter: TORComboBox;
    lstSelected: TListBox;
    tvwService: TTreeView;
    btnConsultAll: TButton;
    procedure lstSelectedKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure cboAvailableChange(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure cboAvailableDblClick(Sender: TObject);
    procedure popListDeleteClick(Sender: TObject);
    procedure popListPropertiesClick(Sender: TObject);
    procedure cboAvailableNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure cboFilterChange(Sender: TObject);
    procedure tvwServiceDblClick(Sender: TObject);
    procedure lstSelectedDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lstSelectedDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure btnConsultAllClick(Sender: TObject);
  private
    FItemType: Integer;
    FDGroupList: TStringList;
    procedure AddToSelected(x: string; isConsultAll: boolean = False);
    procedure DeleteFromSelected;
    procedure SetupContent(ItemType: Integer);
    procedure HideFilter(AllowOffSet: boolean = True);
    procedure ShowFilter;
    procedure SetPropertiesProvider;
    procedure SetPropertiesClinic;
    procedure SetPropertiesDocument;
    procedure SetPropertiesOrderable;
    procedure SetPropertiesConsult;
    procedure SetPropertiesProcedure;
    procedure LoadDGroups(const Parent: string; Node: TTreeNode);
  public
    class procedure InvokeEditor(var ActiveText: string; ValueList: TStringList;
      ItemType: Integer);
  end;

implementation

{$R *.DFM}

uses ORFn, rReport, rPatients;

const
  TX_FILTER_ORDERS    = 'Limit Orderable Items to';
  TX_FILTER_DOCUMENTS = 'Limit Document Class to';
  TC_PROVIDER         = 'Provider';
  TX_PROVIDER         = 'Selected Providers';
  TC_CLINIC           = 'Clinic';
  TX_CLINIC           = 'Selected Clinics';
  TC_DOCUMENT         = 'Document Title';
  TX_DOCUMENT         = 'Selected Document Titles';
  TC_ORDERABLE        = 'Orderable Item';
  TX_ORDERABLE        = 'Selected Orderable Items';
  TC_SERVICE          = 'Service';
  TX_SERVICE          = 'Selected Services';
  TX_TYPE_NAME        = 'Type Name or Select from List';
  TX_HIERARCHY        = 'Service Hierarchy';
  TC_CONSULT          = 'Consult Service';
  TX_CONSULT          = 'Selected Consult Services';
  TC_PROCEDURE        = 'Procedure';
  TX_PROCEDURE        = 'Selected Procedures';
  TX_INSTRUCT_DOC     = 'Use ''Limit Document Class'' to list titles from a particular class.';
  TX_INSTRUCT_ORD     = 'Use ''Limit Orderable Items'' to list subsets of orderable items.';

function FilemanFile(AnItemType: Integer): string;
begin
  Result := '';
  case AnItemType of
  ITM_PROVIDER  : Result := '200';
  ITM_CLINIC    : Result := '44';
  ITM_DOCUMENT  : Result := '8925.1';
  ITM_ORDERABLE : Result := '101.43';
  ITM_SERVICE   : Result := '100.98';
  ITM_CONSULT   : Result := '101.43';
  ITM_PROCEDURE : Result := '101.43';
  end;
end;

class procedure TfrmMultiList.InvokeEditor(var ActiveText: string; ValueList: TStringList;
  ItemType: Integer);
var
  i: Integer;
  ADelim: string;
  frmMultiList: TfrmMultiList;
  ExternalValues: TStringList;
begin
  frmMultiList := TfrmMultiList.Create(Application);
  try
    ResizeAnchoredFormToFont(frmMultiList);
    frmMultiList.SetupContent(ItemType);
    if (ItemType = ITM_CONSULT) and (AnsiCompareText(ActiveText,'all services')=0 ) then
      frmMultiList.btnConsultAllClick(nil)
    else
    begin
      ExternalValues := TStringList.Create;
      ExternalValues.Assign(ValueList);
      LoadExternalValues(ExternalValues, FilemanFile(ItemType));
      for i := 0 to Pred(ExternalValues.Count) do
      begin
        if CharAt(ExternalValues[i], 1) <> '0' then frmMultiList.AddToSelected(ExternalValues[i]);
      end;
      ExternalValues.Free;
    end;
    if frmMultiList.ShowModal = mrOK then
    begin
      ADelim := '';
      ActiveText := '';
      ValueList.Clear;
      for i := 0 to Pred(frmMultiList.lstSelected.Items.Count) do
      begin
        if i > 0 then ADelim := ' or ';
        ValueList.Add(Piece(frmMultiList.lstSelected.Items[i], #9, 2));
        ActiveText := ActiveText + ADelim + Piece(frmMultiList.lstSelected.Items[i], #9, 1);
      end;
    end;
  finally
    frmMultiList.Release;
  end;
end;

procedure TfrmMultiList.SetupContent(ItemType: Integer);
begin
  FItemType := ItemType;
  if ItemType = ITM_SERVICE then
  begin
    FDGroupList := TStringList.Create;
    ListDGroupAll(FDGroupList);
    LoadDGroups('0', nil);
    FDGroupList.Free;
    lblAvailable.Caption := TX_HIERARCHY;
    Self.Caption         := TC_SERVICE;
    lblSelected.Caption  := TX_SERVICE;
    tvwService.FullCollapse;
    tvwService.TopItem   := tvwService.Items[0];
    tvwService.TopItem.Expand(False);
    tvwService.Visible   := TRUE;
    cboAvailable.Visible := FALSE;
    lblDblClick.Visible  := FALSE;
    HideFilter;
  end else
  begin
    lblAvailable.Caption := TX_TYPE_NAME;
    tvwService.Visible   := FALSE;
    cboAvailable.Visible := TRUE;
    if ItemType = ITM_PROVIDER  then SetPropertiesProvider;
    if ItemType = ITM_CLINIC    then SetPropertiesClinic;
    if ItemType = ITM_DOCUMENT  then SetPropertiesDocument;
    if ItemType = ITM_ORDERABLE then SetPropertiesOrderable;
    if ItemType = ITM_CONSULT   then SetPropertiesConsult;
    if ItemType = ITM_PROCEDURE then SetPropertiesProcedure;
    cboAvailable.InitLongList('');
  end;
end;

procedure TfrmMultiList.HideFilter(allowOffSet: boolean);
const
  PIX_OFFSET = 41;
var
  i: Integer;
begin
  lblFilterInstruct.Visible := False;
  lblFilter.Visible := FALSE;
  cboFilter.Visible := FALSE;
  for i := 0 to Pred(Self.ControlCount) do
  begin
    if Self.Controls[i].Tag > 5 then
    begin
     if allowOffSet then
       Self.Controls[i].Top    := Self.Controls[i].Top    - PIX_OFFSET;
    end;
    if Self.Controls[i].Tag = 7 then
    begin
      if allowOffSet then
        Self.Controls[i].Height := Self.Controls[i].Height + PIX_OFFSET;
    end;
  end;
end;

procedure TfrmMultiList.ShowFilter;
begin
  lblFilterInstruct.Visible := True;
  lblFilter.Visible := TRUE;
  cboFilter.Visible := TRUE;
end;

procedure TfrmMultiList.SetPropertiesProvider;
begin
  Self.Caption        := TC_PROVIDER;
  lblSelected.Caption := TX_PROVIDER;
  HideFilter;
end;

procedure TfrmMultiList.SetPropertiesClinic;
begin
  Self.Caption        := TC_CLINIC;
  lblSelected.Caption := TX_CLINIC;
  HideFilter;
end;

procedure TfrmMultiList.SetPropertiesDocument;
begin
  Self.Caption        := TC_DOCUMENT;
  lblSelected.Caption := TX_DOCUMENT;
  lblFilter.Caption := TX_FILTER_DOCUMENTS;
  LoadDocumentClasses(cboFilter.Items);
  cboFilter.SelectByIEN(3);
  ShowFilter;
  lblFilterInstruct.Caption := TX_INSTRUCT_DOC;
end;

procedure TfrmMultiList.SetPropertiesOrderable;
begin
  Self.Caption        := TC_ORDERABLE;
  lblSelected.Caption := TX_ORDERABLE;
  lblFilter.Caption := TX_FILTER_ORDERS;
  LoadOrderableItemFilters(cboFilter.Items);
  ShowFilter;
  lblFilterInstruct.Caption := TX_INSTRUCT_ORD;
end;

procedure TfrmMultiList.SetPropertiesConsult;
begin
  Self.Caption        := TC_CONSULT;
  lblSelected.Caption := TX_CONSULT;
  HideFilter(False);
  btnConsultAll.Visible := True;
end;

procedure TfrmMultiList.SetPropertiesProcedure;
begin
  Self.Caption        := TC_PROCEDURE;
  lblSelected.Caption := TX_PROCEDURE;
  HideFilter;
end;

procedure TfrmMultiList.LoadDGroups(const Parent: string; Node: TTreeNode);
var
  MyID, MyParent, Name: string;
  i, IEN: Integer;
  ChildNode: TTreeNode;
  HasChildren: Boolean;
begin
  with FDGroupList do for i := 0 to Count - 1 do
  begin
    MyParent := Piece(Strings[i], U, 3);
    if MyParent = Parent then
    begin
      MyID := Piece(Strings[i], U, 1);
      IEN  := StrToIntDef(MyID, 0);
      Name := Piece(Strings[i], U, 2);
      HasChildren := Piece(Strings[i], U, 4) = '+';
      ChildNode := tvwService.Items.AddChildObject(Node, Name, Pointer(IEN));
      if HasChildren then LoadDGroups(MyID, ChildNode);
    end;
  end;
end;

procedure TfrmMultiList.cboAvailableChange(Sender: TObject);
begin
  if cboAvailable.ItemIndex > -1 then
  begin
    btnAdd.Default := True;
    btnOK.Default := False;
  end;
end;

procedure TfrmMultiList.cboAvailableDblClick(Sender: TObject);
begin
  btnAddClick(Self);
end;

procedure TfrmMultiList.cboAvailableNeedData(Sender: TObject; const StartFrom: String;
  Direction, InsertAt: Integer);
begin
  if FItemType = ITM_PROVIDER
    then cboAvailable.ForDataUse(SubSetOfPersons(   StartFrom, Direction));
  if FItemType = ITM_CLINIC
    then cboAvailable.ForDataUse(SubSetOfClinics(   StartFrom, Direction));
  if FItemType = ITM_ORDERABLE
    then cboAvailable.ForDataUse(SubSetOfOrderables(StartFrom, Direction, cboFilter.ItemID));
  if FItemType = ITM_DOCUMENT
    then cboAvailable.ForDataUse(SubSetOfDocuments( StartFrom, Direction, cboFilter.ItemIEN));
  if FItemType = ITM_CONSULT
    then cboAvailable.ForDataUse(SubSetOfOrderables(StartFrom, Direction, 'CSLT'));
  if FItemType = ITM_PROCEDURE
    then cboAvailable.ForDataUse(SubSetOfOrderables(StartFrom, Direction, 'PROC'));
end;

procedure TfrmMultiList.tvwServiceDblClick(Sender: TObject);
begin
  //btnAddClick(Self);
end;

procedure TfrmMultiList.btnAddClick(Sender: TObject);
var
  x: string;
  Node: TTreeNode;
begin
  if FItemType = ITM_SERVICE then
  begin
    Node := tvwService.Selected;
    if Node <> nil then
    begin
      x := IntToStr(Integer(Node.Data)) + '^' + Node.Text;
      AddToSelected(x);
      tvwService.Selected := nil;
    end;
  end else
  begin
    if cboAvailable.ItemIndex < 0 then Exit;
    AddToSelected(cboAvailable.Items[cboAvailable.ItemIndex]);
    cboAvailable.ItemIndex := -1;
  end;
  btnAdd.Default := False;
  btnOK.Default := True;
  btnOK.Enabled := lstSelected.Items.Count > 0;
end;

procedure TfrmMultiList.AddToSelected(x: string; isConsultAll: boolean);
var
  i: Integer;
  AnIEN: Int64;
  ConsultGrp: string;
  redundant: Boolean;
begin
  ConsultGrp := GetCousultGrp;
  ConsultGrp := Piece(ConsultGrp,'^',1) + #9 + Piece(ConsultGrp,'^',2);
  if isConsultAll then
  begin
    lstSelected.Items.Clear;
    lstSelected.Items.Add(ConsultGrp);
  end;

  for i := 0 to Pred(lstSelected.Items.Count) do
     if CompareStr(ConsultGrp, lstSelected.Items[i]) = 0 then Exit;

  AnIEN := StrToInt64Def(Piece(x, '^', 1), 0);
  x := ExternalName(AnIEN, StrToFloat(FilemanFile(FItemType))) + #9 + Piece(x, '^', 1);
  redundant := False;
  for i := 0 to Pred(lstSelected.Items.Count) do
    if CompareStr(x, lstSelected.Items[i]) = 0 then redundant := TRUE;
  if not redundant then lstSelected.Items.Add(x);
end;

procedure TfrmMultiList.DeleteFromSelected;
var
  i: Integer;
begin
  for i := Pred(lstSelected.Items.Count) downto 0 do
  begin
    if lstSelected.Selected[i] then lstSelected.Items.Delete(i);
  end;
  btnOK.Enabled := lstSelected.Items.Count > 0;
end;

procedure TfrmMultiList.lstSelectedKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_DELETE then DeleteFromSelected;
end;

procedure TfrmMultiList.popListDeleteClick(Sender: TObject);
begin
  DeleteFromSelected;
end;

procedure TfrmMultiList.popListPropertiesClick(Sender: TObject);
begin
  //TODO: implement properties call
  Application.MessageBox('not implemented', 'not implemented', 0);
end;

procedure TfrmMultiList.cboFilterChange(Sender: TObject);
begin
  if FItemType = ITM_ORDERABLE then cboAvailable.InitLongList('');
  if FItemType = ITM_DOCUMENT  then cboAvailable.InitLongList('');
end;

procedure TfrmMultiList.btnOKClick(Sender: TObject);
begin
  Self.ModalResult := mrOK;
end;

procedure TfrmMultiList.btnCancelClick(Sender: TObject);
begin
  Self.ModalResult := mrCancel;
end;

procedure TfrmMultiList.lstSelectedDragDrop(Sender, Source: TObject; X,
  Y: Integer);
begin
    btnAddClick(Source);
end;

procedure TfrmMultiList.lstSelectedDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if Source is TTreeView then Accept := True;
end;

procedure TfrmMultiList.btnConsultAllClick(Sender: TObject);
var
  ConsultGrp: string;
begin
  ConsultGrp:= GetCousultGrp;
  AddToSelected(ConsultGrp,True);
  btnOK.Enabled := lstSelected.Items.Count > 0;
end;

end.
