unit fPageCriteria;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, CheckLst, mQuery, ExtCtrls, ComCtrls;

type
  TfrmPageCriteria = class(TForm)
    Label2: TLabel;
    Splitter1: TSplitter;
    pnlQuery: TPanel;
    Label1: TLabel;
    fraNewQuery: TfraQuery;
    ProgressBar1: TProgressBar;
    Label3: TLabel;
    shpWizard: TShape;
    sbxWizard: TScrollBox;
    grpScreen: TGroupBox;
    chkScreenProvider: TCheckBox;
    radPtVisitSelect: TRadioButton;
    radPtVisitAny: TRadioButton;
    chkScreenVisit: TCheckBox;
    grpSearchItems: TGroupBox;
    chkOrders: TCheckBox;
    chkConsults: TCheckBox;
    chkDocuments: TCheckBox;
    chkVisits: TCheckBox;
    pnlButtons: TPanel;
    btnNext: TButton;
    btnBack: TButton;
    btnCancel: TButton;
    btnFinish: TButton;
    procedure btnCancelClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
    procedure CriterionClick(Sender: TObject);
    procedure chkScreenVisitClick(Sender: TObject);
    procedure radPtVisitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure chkScreenProviderClick(Sender: TObject);
  private
    FCreating: Boolean;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses ORFn, fHost, rReport, uReport, fPageSaveTo;

const
  TX_REQ_ITEM = 'At least one item must be selected in order to build a report.' + CRLF +
                'Click "Cancel" if you do not wish to build a custom report.';
  TC_REQ_ITEM = 'No Items Selected';

{ Private Procedures and Events ------------------------------------------------------------ }

{ Create and Destroy ----------------------------------------------------------------------- }

procedure TfrmPageCriteria.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  FCreating := True;
  if frmHost.DriveByEdit then
    btnFinish.Enabled := True;
  for i := 0 to Pred(ComponentCount) do
  begin
    if Components[i].Tag = 0 then Continue;
    if not ((Components[i] is TCheckBox) or (Components[i] is TRadioButton)) then Continue;
    if Report.ContainsCriterion(Components[i].Tag) then
    begin
      if Components[i] is TCheckBox    then TCheckBox(Components[i]).Checked := True;
      if Components[i] is TRadioButton then TRadioButton(Components[i]).Checked := True;
    end;
  end;
  if radPtVisitAny.Checked or radPtVisitSelect.Checked then chkScreenVisit.Checked := True;
  FCreating := False;
  fraNewQuery.BuildQuery;
end;

{ Control Events --------------------------------------------------------------------------- }

procedure TfrmPageCriteria.chkScreenVisitClick(Sender: TObject);
begin
  if FCreating then Exit;
  if chkScreenVisit.Checked then
  begin
    radPtVisitAny.Checked := True;
  end else
  begin
    radPtVisitAny.Checked    := False;
    radPtVisitSelect.Checked := False;
    Report.RemoveCriterion(radPtVisitAny.Tag);
    Report.RemoveCriterion(radPtVisitSelect.Tag);
    fraNewQuery.BuildQuery;
  end;
end;

procedure TfrmPageCriteria.radPtVisitClick(Sender: TObject);
begin
  if FCreating then Exit;
  chkScreenVisit.Checked := True;
  if radPtVisitAny.Checked then
  begin
    Report.RemoveCriterion(radPtVisitSelect.Tag,True);
    Report.AddCriterion(radPtVisitAny.Tag,True);
  end;
  if radPtVisitSelect.Checked then
  begin
    Report.RemoveCriterion(radPtVisitAny.Tag,True);
    Report.AddCriterion(radPtVisitSelect.Tag,True);
  end;
  fraNewQuery.BuildQuery;
end;

procedure TfrmPageCriteria.CriterionClick(Sender: TObject);
begin
  if FCreating then Exit;
  with Sender as TCheckBox do
  begin
    if Checked
     then Report.AddCriterion(Tag)
     else Report.RemoveCriterion(Tag);
  end;
  fraNewQuery.BuildQuery;
end;

{ Button Events ---------------------------------------------------------------------------- }

procedure TfrmPageCriteria.btnCancelClick(Sender: TObject);
begin
  Report.Clear;
  OldQuery.Clear;
  frmHost.DriveByEdit := False;  
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageCriteria.btnBackClick(Sender: TObject);
begin
  // disabled
end;

procedure TfrmPageCriteria.btnNextClick(Sender: TObject);
begin
  // There will be no query text if no items have been selected
  if ShowMsgOn(Report.QueryText[0] = nil, TX_REQ_ITEM, TC_REQ_ITEM) then Exit;
  // Setup wizard to show appropriate screens
  frmHost.WizardShow[CONTENT_ORDERS]    := Report.ContainsConstraint('Order');
  frmHost.WizardShow[CONTENT_CONSULTS]  := Report.ContainsConstraint('Consult');
  frmHost.WizardShow[CONTENT_DOCUMENTS] := Report.ContainsConstraint('Document');
  frmHost.WizardShow[CONTENT_VISITS]    := Report.ContainsConstraint('Visit');
  frmHost.WizardNext;
end;

procedure TfrmPageCriteria.btnFinishClick(Sender: TObject);
begin
  frmHost.DriveByEdit := False;
  if IsGenericReport(Report.DisplayText) then
    SaveReportTo
  else
    SaveEditedReport;
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageCriteria.chkScreenProviderClick(Sender: TObject);
begin
  if FCreating then Exit;
  with Sender as TCheckBox do
  begin
    if Checked
      then Report.AddCriterion(Tag,True)
      else Report.RemoveCriterion(Tag,True);
  end;
  fraNewQuery.BuildQuery;
end;

end.
