unit fPageDisplay;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, uPtQuery;

const
  UM_SETUP_REPORT = (WM_USER + 9245);

type
  TfrmPageDisplay = class(TForm)
    pnlButtons: TPanel;
    Bevel1: TBevel;
    btnBack: TButton;
    btnPrint: TButton;
    btnExport: TButton;
    Splitter1: TSplitter;
    pnlTitle: TPanel;
    lblTitle: TLabel;
    lblDate: TLabel;
    lvwReport: TListView;
    pnlSearch: TPanel;
    lblPatients: TLabel;
    lblRecords: TLabel;
    lblSearch: TLabel;
    lblPtExcluded: TLabel;
    lblCountRecords: TLabel;
    Label1: TLabel;
    lblPtEvaluated: TLabel;
    btnCancel: TButton;
    pnlDetail: TPanel;
    pnlPtID: TPanel;
    lblPtName: TLabel;
    lblPtBID: TLabel;
    lblRmBed: TLabel;
    Bevel2: TBevel;
    memDetails: TMemo;
    dlgSave: TSaveDialog;
    lblRecordCount: TLabel;
    btnExit: TButton;
    lblWarning: TLabel;
    procedure lvwReportClick(Sender: TObject);
    procedure lvwReportColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvwReportData(Sender: TObject; Item: TListItem);
    procedure lvwReportDataHint(Sender: TObject; StartIndex, EndIndex: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lvwReportKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnExitClick(Sender: TObject);
    procedure lvwReportEnter(Sender: TObject);
  private
    FCancel: Boolean;
    FSortColumn: Integer;
    FSortAscend: Boolean;
    FPtQuery: TPtQuery;
    procedure ClearSelection;
    procedure SetupColumns;
    procedure SetupReport;
    procedure UpdateSearchDisplay(const ACaption: string);
    procedure UMSetupReport(var Message: TMessage); message UM_SETUP_REPORT;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses ORFn, fHost, uReport, rReport;

const
  TX_WARN = 'The Reports exported will contain Confidential Information' +
            CRLF + CRLF + 'Do you wish to continue?';
  TC_WARN = 'Confidential Patient Information';

{ Private Methods -------------------------------------------------------------------------- }

procedure TfrmPageDisplay.ClearSelection;
{ clears the detailed display and associated patient identifying information }
begin
  lvwReport.Selected := nil;
  lblPtName.Caption := 'No Item Selected';
  lblPtBID.Caption  := '';
  lblRmBed.Caption  := '';
  memDetails.Lines.Clear;
end;

procedure TfrmPageDisplay.SetupColumns;
{ set up columns based on report definition, column 0 (hidden) contains ID for the row }
var
  i: Integer;
  AListColumn: TListColumn;
begin
  AListColumn := lvwReport.Columns.Add;
  AListColumn.Caption := 'Internal ID';
  AListColumn.Width := 0;
  for i := 0 to Pred(Report.ColCount) do
  begin
    AListColumn := lvwReport.Columns.Add;
    AListColumn.Caption := Report.ColHeader[i];
    AListColumn.Width := Report.ColWidth[i];
  end;
end;

procedure TfrmPageDisplay.SetupReport;
{ performs the patient screen and report query in iterative fashion to allow interupt }
begin
  SetupColumns;
  FPtQuery.Initiate;
  pnlSearch.Visible := True;
  while not (FCancel or FPtQuery.Finished) do
  begin
    FPtQuery.Iterate;
    UpdateSearchDisplay('Searching...');
    Application.ProcessMessages;
  end;
  UpdateSearchDisplay('Sorting...');
  FPtQuery.SortByColumn(1, TRUE);
  pnlSearch.Visible := False;
  if FPtQuery.RecordCount > 0 then
  begin
    lblRecordCount.Caption := '(' + IntToStr(FPtQuery.RecordCount) + ' records)';
    with lblRecordCount do if FCancel then Caption := Caption + ' QUERY STOPPED';
    lvwReport.Items.Count  := FPtQuery.RecordCount;
    lvwReport.Enabled      := True;
    //MessageDlg(TXT_WARNING,mtWarning,[mbOK],0);    
  end
  else
  begin
    InfoBox('No items were found for this report', 'No Records Found', MB_OK);
    pnlTitle.SetFocus;
  end;
end;

procedure TfrmPageDisplay.UpdateSearchDisplay(const ACaption: string);
{ updates the status counts while the query is being performed }
begin
  lblSearch.Caption := ACaption;
  lblPtEvaluated.Caption := IntToStr(FPtQuery.PatientCount);
  lblPtExcluded.Caption  := IntToStr(FPtQuery.ScreenedCount);
  lblCountRecords.Caption  := IntToStr(FPtQuery.RecordCount);
  pnlSearch.Refresh;
end;

procedure TfrmPageDisplay.UMSetupReport(var Message: TMessage);
{ receives the user window message that delays query execution until after form is shown }
begin
  SetupReport;
end;

{ Form Events ------------------------------------------------------------------------------ }

procedure TfrmPageDisplay.FormCreate(Sender: TObject);
{ set up the report title, default sorting, etc. }
begin
  lblTitle.Caption := Report.DisplayText;
  lblDate.Caption := FormatDateTime('dd mmm yyyy hh:nn', Now);
  FPtQuery := TPtQuery.Create(Report);
  FSortColumn := 1;
  FSortAscend := True;
  FCancel := False;
end;

procedure TfrmPageDisplay.FormDestroy(Sender: TObject);
{ clean up objects used by the form }
begin
  FPtQuery.Cancel;
  FPtQuery.Free;
end;

procedure TfrmPageDisplay.FormShow(Sender: TObject);
{ posting message allows the query to execute after the form is displayed, allowing cancel }
begin
  PostMessage(Self.Handle, UM_SETUP_REPORT, 0, 0);
end;

{ Control Events --------------------------------------------------------------------------- }

procedure TfrmPageDisplay.btnCancelClick(Sender: TObject);
{ cancels the query (may need to wait until RPC returns), and cleans up server globals }
begin
  FCancel := TRUE;
  //frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageDisplay.lvwReportClick(Sender: TObject);
{ brings up appropriate detailed display and shows the associated patient information }
var
  AnID, PtInfo: string;
begin
  if lvwReport.Selected = nil then Exit;
  AnID := lvwReport.Selected.Caption;
  if Piece(AnID, ':', 1) = 'ORD'
    then DetailOrder(memDetails.Lines, Copy(AnID, 5, Length(AnID)));
  if Piece(AnID, ':', 1) = 'DOC'
    then DetailDocument(memDetails.Lines, Copy(AnID, 5, Length(AnID)));
  if (Piece(AnID, ':', 1) <> 'ORD') and (Piece(AnID, ':', 1) <> 'DOC')
    then memDetails.Text := 'No Details Available';
  PtInfo := GetPtInfo(AnID);
  lblPtName.Caption := Piece(PtInfo, U, 1);
  lblPtBID.Caption  := Piece(PtInfo, U, 2);
  lblRmBed.Caption  := Piece(PtInfo, U, 3);
end;

procedure TfrmPageDisplay.lvwReportColumnClick(Sender: TObject; Column: TListColumn);
{ calls the server to resort the selected column and refreshes display }
var
  StartIndex, EndIndex: Integer;
begin
  if FSortColumn = Column.Index then FSortAscend := not FSortAscend else FSortAscend := TRUE;
  FSortColumn := Column.Index;
  FPtQuery.SortByColumn(FSortColumn, FSortAscend);
  StartIndex := lvwReport.TopItem.Index;
  EndIndex := StartIndex + lvwReport.VisibleRowCount + 1;
  lvwReport.UpdateItems(StartIndex, EndIndex);
  ClearSelection;
end;

procedure TfrmPageDisplay.lvwReportData(Sender: TObject; Item: TListItem);
{ adds data for the item to the cache (if not already there) and sets row data }
var
  j: Integer;
begin
  FPtQuery.CacheRowData(Item.Index, Item.Index);
  Item.Caption := FPtQuery.RowID[Item.Index];
  for j := 0 to Pred(Report.ColCount) do
    Item.SubItems.Add(FPtQuery.Cell[j, Item.Index]);
end;

procedure TfrmPageDisplay.lvwReportDataHint(Sender: TObject; StartIndex, EndIndex: Integer);
{ adds data for a series of rows to the cache and sets the row data in the list view }
var
  i, j: Integer;
begin
  FPtQuery.CacheRowData(StartIndex, EndIndex);
  for i := StartIndex to EndIndex do
  begin
    lvwReport.Items[i].Caption := FPtQuery.RowID[i];
    for j := 0 to Pred(Report.ColCount) do
       lvwReport.Items[i].SubItems.Add(FPtQuery.Cell[j, i]);
  end;
end;

{ Lower Buttons ----------------------------------------------------------------------------- }

procedure TfrmPageDisplay.btnPrintClick(Sender: TObject);
begin
  Application.MessageBox('Printing is not yet implemented.', 'Print Report', MB_OK);
end;

procedure TfrmPageDisplay.btnExportClick(Sender: TObject);
begin
  if Application.MessageBox(TX_WARN, TC_WARN, MB_YESNO) = IDYES then
  begin
    if dlgSave.Execute then FPtQuery.ExportToFile(dlgSave.FileName);
  end;
end;

procedure TfrmPageDisplay.btnBackClick(Sender: TObject);
{ cleans up the server query globals before returning to the display of available reports }
begin
  ClearQuery;
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageDisplay.lvwReportKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_RETURN then
    lvwReportClick(Self);
end;

procedure TfrmPageDisplay.btnExitClick(Sender: TObject);
begin
  frmHost.Close;
end;

procedure TfrmPageDisplay.lvwReportEnter(Sender: TObject);
begin
  if lvwReport.Selected = nil then
     lvwReport.Selected := lvwReport.Items[0];
end;

end.
