unit fPageFormat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Buttons, ExtCtrls, ComCtrls;

type
  TfrmPageFormat = class(TForm)
    pnlButtons: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    btnAdd: TButton;
    btnDelete: TButton;
    lstAvailable: TListBox;
    lstSelected: TListBox;
    btnCancel: TButton;
    btnBack: TButton;
    btnNext: TButton;
    btnFinish: TButton;
    Label5: TLabel;
    ProgressBar1: TProgressBar;
    shpWizard: TShape;
    btnUp: TBitBtn;
    btnDown: TBitBtn;
    grdReport: TStringGrid;
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnDown2Click(Sender: TObject);
    procedure btnUp2Click(Sender: TObject);
    procedure lstAvailableDblClick(Sender: TObject);
    procedure grdReportMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnCancelClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lstAvailableKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lstSelectedKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    FLoading: Boolean;
    function FindIndexInternal(const AName: string): Integer;
    procedure RefreshHeader;
    procedure SaveColumns;
    procedure SetupAvailableFields(const FieldTypes: string);
    procedure SynchColumns;
    procedure AddPredefinedColumns;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses ORFn, fHost, uReport, rReport, fpageSaveTo;

const
  FLDS_PATIENT  = 'P';
  FLDS_ORDER    = 'O';
  FLDS_CONSULT  = 'C';
  FLDS_DOCUMENT = 'D';
  FLDS_VISIT    = 'V';
  TX_OVERWRITE  = 'A report with this name already exists.' + CRLF +
                  'Do you wish to overwrite it?';
  TC_OVERWRITE  = 'Overwrite Report?';
  TXT_SAVE = 'Do you want to save the changes you made to the ';

type
  TDisplayField = class
  public
    InternalName: string;
    DisplayName:  string;
    HeaderName:   string;
    Samples: TStringList;
    constructor Create;
    destructor Destroy; override;
  end;

{ TDisplayField ---------------------------------------------------------------------------- }

constructor TDisplayField.Create;
begin
  Samples := TStringList.Create;
end;

destructor TDisplayField.Destroy;
begin
  Samples.Free;
  inherited;
end;

{ Private Procedures and Functions --------------------------------------------------------- }

function TfrmPageFormat.FindIndexInternal(const AName: string): Integer;
var
  i: Integer;
  ADisplayField: TDisplayField;
begin
  Result := -1;
  for i := 0 to Pred(lstAvailable.Items.Count) do
  begin
    ADisplayField := TDisplayField(lstAvailable.Items.Objects[i]);
    if ADisplayField.InternalName = AName then
    begin
      Result := i;
      Break;
    end; {if ADisplayField}
  end; {for i}
end;

procedure TfrmPageFormat.RefreshHeader;
var
  i, j: Integer;
  ADisplayField: TDisplayField;
begin
  if lstSelected.Items.Count = 0 then
  begin
    grdReport.Cells[0,0] := '';
    grdReport.Cells[0,1] := '';
    grdReport.Cells[0,2] := '';
    grdReport.Cells[0,3] := ''
  end;
  grdReport.ColCount := lstSelected.Items.Count;
  for i := 0 to Pred(lstSelected.Items.Count) do
  begin
    ADisplayField := TDisplayField(lstSelected.Items.Objects[i]);
    grdReport.Cells[i, 0] := ADisplayField.HeaderName;
    for j := 0 to Pred(ADisplayField.Samples.Count) do
      grdReport.Cells[i, j+1] := ADisplayField.Samples[j];
  end;
  SaveColumns;
end;

procedure TfrmPageFormat.SaveColumns;
var
  i: Integer;
  ADisplayField: TDisplayField;
begin
  if FLoading then Exit;
  Report.ColCount := lstSelected.Items.Count;
  for i := 0 to Pred(lstSelected.Items.Count) do
  begin
    ADisplayField := TDisplayField(lstSelected.Items.Objects[i]);
    Report.ColName[i]   := ADisplayField.InternalName;
    Report.ColHeader[i] := grdReport.Cells[i, 0];
    Report.ColWidth[i]  := grdReport.ColWidths[i];
  end;
end;

procedure TfrmPageFormat.SetupAvailableFields(const FieldTypes: string);
var
  i: Integer;
  NameValueList: TStringList;
  ADisplayField: TDisplayField;
begin
  NameValueList := TStringList.Create;
  LoadDisplayFields(NameValueList, FieldTypes);
  ADisplayField := nil;
  for i := 0 to Pred(NameValueList.Count) do
  begin
    if NameValueList.Names[i] = 'DisplayName' then
    begin
      ADisplayField := TDisplayField.Create;
      ADisplayField.DisplayName := ValuePart(NameValueList[i]);
      lstAvailable.Items.AddObject(ADisplayField.DisplayName, ADisplayField);
    end;
    if ADisplayField = nil then Continue;
    if NameValueList.Names[i] = 'InternalName' then
      ADisplayField.InternalName := ValuePart(NameValueList[i]);
    if NameValueList.Names[i] = 'HeaderName' then
      ADisplayField.HeaderName := ValuePart(NameValueList[i]);
    if NameValueList.Names[i] = 'SampleData' then
      ADisplayField.Samples.Add(ValuePart(NameValueList[i]));
  end;
  NameValueList.Free;
end;

procedure TfrmPageFormat.SynchColumns;
var
  i, AnIndex: Integer;
  ColNames: TStringList;
begin
  ColNames := TStringList.Create;
  for i := 0 to Pred(Report.ColCount) do
    ColNames.AddObject(Report.ColName[i], Pointer(Report.ColWidth[i]));
  Report.ColCount := 0;
  for i := 0 to Pred(ColNames.Count) do
  begin
    AnIndex := FindIndexInternal(ColNames[i]);
    if AnIndex > -1 then
    begin
      lstAvailable.ItemIndex := AnIndex;
      AddPredefinedColumns;
      grdReport.ColWidths[grdReport.ColCount-1] := Integer(ColNames.Objects[i]);
    end;
  end;
end;

{ Create and Destroy ----------------------------------------------------------------------- }

procedure TfrmPageFormat.FormCreate(Sender: TObject);
var
  FieldTypes: string;
begin
  FLoading := True;
  FieldTypes := '';
  if Report.ContainsConstraint('Patient')  then FieldTypes := FieldTypes + FLDS_PATIENT;
  if Report.ContainsConstraint('Order')
    and not Report.ContainsConstraint('Order.NegativeSearch')
      then FieldTypes := FieldTypes + FLDS_ORDER;
  if Report.ContainsConstraint('Consult')
    and not Report.ContainsConstraint('Consult.NegativeSearch')
      then FieldTypes := FieldTypes + FLDS_CONSULT;
  if Report.ContainsConstraint('Document')
    and not Report.ContainsConstraint('Document.NegativeSearch')
      then FieldTypes := FieldTypes + FLDS_DOCUMENT;
  if Report.ContainsConstraint('Visit')
    and not Report.ContainsConstraint('Visit.NegativeSearch')
      then FieldTypes := FieldTypes + FLDS_VISIT;
  SetupAvailableFields(FieldTypes);
  SynchColumns;
  FLoading := False;
  RefreshHeader;
end;

procedure TfrmPageFormat.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  for i := Pred(lstAvailable.Items.Count) downto 0 do
    TDisplayField(lstAvailable.Items.Objects[i]).Free;
end;

{ Control Events --------------------------------------------------------------------------- }

procedure TfrmPageFormat.lstAvailableDblClick(Sender: TObject);
begin
  btnAddClick(Self);
end;

procedure TfrmPageFormat.btnAddClick(Sender: TObject);
var
  i,j,ix: Integer;
  x: string;
  ADisplayField: TDisplayField;
  Redundant: Boolean;
begin
  with lstAvailable do for i := 0 to Pred(Items.Count) do
  begin
    if Selected[i] then
    begin
      x := Items[i];
      Redundant := False;
      for j := 0 to Pred(lstSelected.Items.Count) do
        if CompareStr(x,lstSelected.Items[j])=0 then redundant := True;
      if not Redundant then
      begin
        ix := -1;
        ADisplayField := TDisplayField.Create;
        ix := lstAvailable.Items.IndexOf(x);
        if ix > -1 then
        begin
          ADisplayField := TDisplayField(lstAvailable.Items.Objects[ix]);
          lstSelected.Items.AddObject(x, ADisplayField);
        end;
      end;
    end;
  end;
  RefreshHeader;
end;

procedure TfrmPageFormat.btnDeleteClick(Sender: TObject);
var
  i: Integer;
begin
  with lstSelected do for i := Pred(items.Count) downto 0 do
    if Selected[i] then Items.Delete(i);
  RefreshHeader;
end;

procedure TfrmPageFormat.btnDown2Click(Sender: TObject);
var
  NewIndex: Integer;
begin
  if lstSelected.Items.Count < 1 then Exit;
  if lstSelected.ItemIndex < 0 then Exit;
  NewIndex := LowerOf(lstSelected.ItemIndex + 1, lstSelected.Items.Count - 1);
  with lstSelected do Items.Move(ItemIndex, NewIndex);
  lstSelected.ItemIndex := NewIndex;
  RefreshHeader;
end;

procedure TfrmPageFormat.btnUp2Click(Sender: TObject);
var
  NewIndex: Integer;
begin
  if lstSelected.ItemIndex < 0 then Exit;
  NewIndex := HigherOf(lstSelected.ItemIndex - 1, 0);
  with lstSelected do Items.Move(ItemIndex, NewIndex);
  lstSelected.ItemIndex := NewIndex;
  RefreshHeader;
end;

procedure TfrmPageFormat.grdReportMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  SaveColumns;
end;

{ Navigation Buttons ----------------------------------------------------------------------- }

procedure TfrmPageFormat.btnCancelClick(Sender: TObject);
begin
  Report.Clear;
  OldQuery.Clear;
  frmHost.DriveByEdit := False;  
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageFormat.btnBackClick(Sender: TObject);
begin
  frmHost.WizardBack;
end;

procedure TfrmPageFormat.btnNextClick(Sender: TObject);
begin
  if lstSelected.Items.Count = 0 then
  begin
    ShowMessage('Please select at least one report field before continuing!');
    Exit;
  end;
  frmHost.WizardNext;
end;

procedure TfrmPageFormat.btnFinishClick(Sender: TObject);
const
  TX_SAVE1 = 'Do you want to save the report ';
  TX_SAVE2 = ' for later reuse?';
begin
  if lstSelected.Items.Count = 0 then
  begin
    ShowMessage('Please select one or serveral reports'' fields before finishing it.');
    Exit;
  end;
  if frmHost.DriveByEdit then
  begin
    if IsGenericReport(Report.DisplayText) then SaveReportTo
    else SaveEditedReport;
    frmHost.DriveByEdit := False;
  end else
  begin
    if MessageDlg( TX_SAVE1 + Report.DisplayText + TX_SAVE2, mtWarning, [mbYes, mbNo], 0) = mrYes then
      Report.Save(True)
    else
      Exit;
  end;

  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageFormat.AddPredefinedColumns; 
var
  i: Integer;
  x: string;
  Redundant: Boolean;
begin
  if lstAvailable.ItemIndex < 0 then Exit;
  x := lstAvailable.Items[lstAvailable.ItemIndex];
  Redundant := False;
  for i := 0 to Pred(lstSelected.Items.Count) do
    if CompareStr(x, lstSelected.Items[i]) = 0 then redundant := TRUE;
  if not redundant
    then lstSelected.Items.AddObject(x, lstAvailable.Items.Objects[lstAvailable.ItemIndex]);
  RefreshHeader;
end;

procedure TfrmPageFormat.lstAvailableKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then
     btnAddClick(Self);
end;

procedure TfrmPageFormat.lstSelectedKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
    btnDeleteClick(Self);
end;

end.
