unit fPageReports;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  mQuery, StdCtrls, ORCtrls, ExtCtrls,  uReport, Menus, ShellApi;

type
  TfrmPageReports = class(TForm)
    pnlButtons: TPanel;
    Bevel1: TBevel;
    btnEdit: TButton;
    btnDelete: TButton;
    btnReport: TButton;
    lblQuery: TLabel;
    Splitter1: TSplitter;
    pnlReport: TPanel;
    lblReport: TLabel;
    lstReports: TORListBox;
    pnlQuery: TPanel;
    lblQueryInstruct1: TLabel;
    lblQueryInstruct3: TLabel;
    lblQueryInstruct2: TLabel;
    fraRptQuery: TfraQuery;
    btnRename: TButton;
    popMenu1: TPopupMenu;
    Edit1: TMenuItem;
    Delete1: TMenuItem;
    Rename1: TMenuItem;
    btnCancel: TButton;
    ShowReport1: TMenuItem;
    btnSaveAs: TButton;
    mnuReport: TMainMenu;
    mnReport: TMenuItem;
    mnEdit: TMenuItem;
    mnDelete: TMenuItem;
    mnRename: TMenuItem;
    mnSaveas: TMenuItem;
    mnExit: TMenuItem;
    mnHelp: TMenuItem;
    mnAbout: TMenuItem;
    procedure btnEditClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnReportClick(Sender: TObject);
    procedure lstReportsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnRenameClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveAsClick(Sender: TObject);
    procedure mnAboutClick(Sender: TObject);
  private
    procedure SetToReport(AnIEN: Integer);
    procedure PopulateRPTList;
    procedure ControlActions(EnableAct: boolean);
  public
    { Public declarations }
  end;

implementation

uses ORFn, fHost, uCohort,fPageSaveTo,fAbout;

const
  ID_CUSTOM_REPORT = 'CREATE';

  TX_NO_VALUES = 'The report cannot be built until values for all criteria are set.' + CRLF +
                 'Click the blue links in the query to add necessary values.';
  TC_NO_VALUES = 'Missing Values';
  TX_NO_REPORT = 'A report must first be selected or created.';
  TC_NO_REPORT = 'No Report Selected';
  TX_NO_DELETE = 'This report may not be deleted.  ' + CRLF +
                 'Only personally created reports may be deleted.';
  TC_NO_DELETE = 'No Deletion';
  TX_NO_EDIT   = 'This report may not be edited.  ' + CRLF +
                 'Only personally created reports may be edited.';
  TC_NO_EDIT   = 'No Editing';
  TX_CHK_DEL1  = 'Report: ';
  TX_CHK_DEL2  = CRLF + 'Do you wish to delete this report?';
  TC_CHK_DEL   = 'Delete Report';

{$R *.DFM}

{ Create & Destroy ------------------------------------------------------------------------- }

procedure TfrmPageReports.FormCreate(Sender: TObject);
{ set up form to reflect currently active report and list all reports available from server }
begin
  PopulateRPTList;
end;

procedure TfrmPageReports.SetToReport(AnIEN: Integer);
{ set up the report and cohort objects and screen to reflect the active report }
var
  PatientSourceValues: TStringList;
begin
  PatientSourceValues := TStringList.Create;
  Report.Id := AnIEN;
  Report.ValuesByConstraint(PatientSourceValues, 'Patient.ListSource');
  Cohort.Clear;
  Cohort.SetFromValueList(PatientSourceValues);
  fraRptQuery.BuildQuery;
  PatientSourceValues.Free;
end;

{ List Events ------------------------------------------------------------------------------ }

procedure TfrmPageReports.lstReportsClick(Sender: TObject);
{ load the new report definition, or check for special IEN that starts custom report wizard }
begin
  if lstReports.ItemID = '0' then
  begin
    lstReports.ItemIndex := -1;
    Exit;
  end;
  ControlActions(True);
  if lstReports.ItemIEN <> 0 then SetToReport(lstReports.ItemIEN);
  if (lstReports.ItemID <> ID_CUSTOM_REPORT) and (Pos('RPTU',Report.Name)=1) then
  begin
    if OldQuery.Count > 0 then OldQuery.Clear;
    Report.QueryToStrings(OldQuery);
  end else OldQuery.Clear;
  if lstReports.ItemID = ID_CUSTOM_REPORT then
  begin
    Cohort.Clear;
    Report.Id := 0;
    frmHost.SwitchTo(CONTENT_CRITERIA);
  end;
  if IsGenericReport(Report.DisplayText) then
  begin
    mnRename.Enabled := False;
    btnRename.Enabled := False;
    Rename1.Enabled := False;
  end
  else
  begin
    mnRename.Enabled := True;
    btnRename.Enabled := True;
    Rename1.Enabled := True;
  end;
end;

{ Button Events ---------------------------------------------------------------------------- }

procedure TfrmPageReports.btnEditClick(Sender: TObject);
begin
  if ShowMsgOn(Report.Id = 0, TX_NO_REPORT, TC_NO_REPORT) then Exit;
  frmHost.DriveByEdit := True;
  frmHost.SwitchTo(CONTENT_CRITERIA);
end;

procedure TfrmPageReports.btnDeleteClick(Sender: TObject);
var
  x: string;
begin
  if ShowMsgOn(Report.Id = 0, TX_NO_REPORT, TC_NO_REPORT) then Exit;
  if ShowMsgOn(not Report.IsOwner, TX_NO_DELETE, TC_NO_DELETE) then Exit;
  x := TX_CHK_DEL1 + Report.DisplayText + TX_CHK_DEL2;
  if Application.MessageBox(PChar(x), TC_CHK_DEL, MB_YESNO) = IDYES then
  begin
    Report.Delete;
    OldQuery.Clear;
    frmHost.SwitchTo(CONTENT_REPORTS);  //TODO: just reload listbox instead?
  end;
end;

procedure TfrmPageReports.btnReportClick(Sender: TObject);
begin
  if ShowMsgOn(Report.ValuesAreMissing, TX_NO_VALUES, TC_NO_VALUES) then Exit;
  if ShowMsgOn(Report.Id = 0,           TX_NO_REPORT, TC_NO_REPORT) then Exit;
  if Trim(Copy(Report.Name,1,4)) <> 'RPT' then
    SaveEditedReport;
  frmHost.SwitchTo(CONTENT_DISPLAY);
end;

procedure TfrmPageReports.btnRenameClick(Sender: TObject);
begin
  frmHost.DriveByEdit := True;
  if ShowMsgOn(Report.Id = 0, TX_NO_REPORT, TC_NO_REPORT) then Exit;
  frmHost.SwitchTo(CONTENT_SAVERPT);
end;

procedure TfrmPageReports.btnCancelClick(Sender: TObject);
begin
  frmHost.Close;
end;

procedure TfrmPageReports.btnSaveAsClick(Sender: TObject);
begin
 if Length(Report.Name)>0 then
 begin
    SaveReportTo;
    PopulateRPTList;
 end;
end;

procedure TfrmPageReports.PopulateRPTList;
var
  i: Integer;
begin
  lstReports.Items.Clear;
  Report.ListNamesByUser(lstReports.Items);
  for i := 0 to Pred(lstReports.Items.Count) do
    if Piece(lstReports.Items[i], U, 1) = '0' then break;
  lstReports.Items.Insert(i, 'CREATE^Create Custom Search...');
  lstReports.Items.Insert(i, '0^ ');
  if (OldQuery <> nil) and ( OldQuery.Count > 0) then
    OldQuery.Clear;
  ControlActions(False);  
  if Report.Id <> 0 then
  begin
    ControlActions(True);
    lstReports.SelectByIEN(Report.Id);
    SetToReport(Report.Id);
    Report.QueryToStrings(OldQuery);
    if IsGenericReport(Report.DisplayText) then
    begin
      btnRename.Enabled := False;
      mnRename.Enabled := False;
      Rename1.Enabled := False;
    end
    else
    begin
      mnRename.Enabled := True;
      btnRename.Enabled := True;
      Rename1.Enabled := True;
    end;
  end else
  begin
    lstReports.ItemIndex := 0;
    lstReportsClick(Self);
  end;
end;

procedure TfrmPageReports.ControlActions(EnableAct: boolean);
begin
  btnEdit.Enabled     := EnableAct;
  btnDelete.Enabled   := EnableAct;
  btnRename.Enabled   := EnableAct;
  btnSaveAs.Enabled   := EnableAct;
  mnEdit.Enabled      := EnableAct;
  mnDelete.Enabled    := EnableAct;
  mnRename.Enabled    := EnableAct;
  mnSaveas.Enabled    := EnableAct;
end;

procedure TfrmPageReports.mnAboutClick(Sender: TObject);
begin
  ShowAbout;
end;

end.
