unit fPageSave;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons;

type
  TfrmPageSave = class(TForm)
    pnlButtons: TPanel;
    btnCancel: TButton;
    btnBack: TButton;
    btnNext: TButton;
    btnFinish: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Label5: TLabel;
    Label1: TLabel;
    Label6: TLabel;
    txtRptName: TEdit;
    chkServer: TCheckBox;
    Label4: TLabel;
    ProgressBar1: TProgressBar;
    shpWizard: TShape;
    btnCurrentRpt: TBitBtn;
    lstCurrReports: TListBox;
    procedure txtRptNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnBackClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnFinishClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnCurrentRptClick(Sender: TObject);
    procedure txtRptNameKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    FExistRPTList: TStringList;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses ORFn, fHost, uReport, rReport;

const
  TX_OVERWRITE = 'A report with this name already exists.' + CRLF +
                 'Do you wish to overwrite it?';
  TC_OVERWRITE = 'Overwrite Report?';

{ Create and Destroy ----------------------------------------------------------------------- }

procedure TfrmPageSave.FormCreate(Sender: TObject);
begin
  txtRptName.Text := Report.DisplayText;
  FExistRPTList := TStringList.Create;
  Report.ListNamesByUser(FExistRPTList);
  if Report.Id > 0 then chkServer.Checked := True else chkServer.Checked := False;
end;

procedure TfrmPageSave.FormDestroy(Sender: TObject);
begin
  FExistRPTList.Free;
end;

{ Control Events --------------------------------------------------------------------------- }

procedure TfrmPageSave.txtRptNameChange(Sender: TObject);
begin
//
end;

{ Navigation Button Events ----------------------------------------------------------------- }

procedure TfrmPageSave.btnCancelClick(Sender: TObject);
begin
  Report.Clear;
  OldQuery.Clear;
  frmHost.DriveByEdit := False;
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageSave.btnBackClick(Sender: TObject);
begin
  frmHost.SwitchTo(CONTENT_FORMAT);
end;

procedure TfrmPageSave.btnNextClick(Sender: TObject);
begin
  // disabled
end;

procedure TfrmPageSave.btnFinishClick(Sender: TObject);
const
  WTXT1 = 'The report name ';
  WTXT2 = ' is currently used by another report.' + #13 + 'Please re-enter a new report name.';
  TX_NO_UPARR  = 'The report name is not valid. You can not use "^" in a report name.';
  TC_NO_UPARR  = 'Invalid Report Name';

var
  i: integer;
  preName: string;
begin
  preName := Report.DisplayText;
  if ShowMsgOn(Length(txtRPTName.Text)<1,'Please enter the report name.','Missing report name') then
  begin
    ActiveControl := txtRPTName;
    Exit;
  end;

  if ShowMsgOn((Pos('^',txtRPTName.Text)>0),TX_NO_UPARR, TC_NO_UPARR) then
  begin
    txtRPTName.Text := '';
    ActiveControl := txtRPTName;
    Exit;
  end;

  Report.DisplayText := txtRPTName.Text;

  for i := 0 to FExistRPTList.Count - 1 do
  begin
    if CompareStr(Report.DisplayText,Piece(FExistRPTList[i],'^',2))=0 then
    begin
      if ShowMsgOn(IsGenericReport(Report.DisplayText),WTXT1 + Report.DisplayText + WTXT2,'Duplicated report name') then
      begin
        txtRPTName.Text := '';
        txtRPTName.SetFocus;
        Exit;
      end;
      if (preName=Report.DisplayText) and frmHost.DriveByEdit then break;
      if ShowMsgOn(True,WTXT1 + Report.DisplayText + WTXT2,'Duplicated report name') then
      begin
        txtRPTName.Text := '';
        txtRPTName.SetFocus;
        Exit;
      end;
    end;
  end;

  if frmHost.DriveByEdit and Report.ExistsOnServer then
  begin
    if chkServer.Checked then
      SaveEditedReport
    else
      Report.Save(False);
    frmHost.DriveByEdit := False;
  end else
  begin
    Report.Save(chkServer.Checked);
  end;
  frmHost.SwitchTo(CONTENT_REPORTS);
end;

procedure TfrmPageSave.btnCurrentRptClick(Sender: TObject);
var
  i: integer;
begin
  lstCurrReports.Clear;
  lstCurrReports.Visible := not  lstCurrReports.Visible;
  if lstCurrReports.Visible then
  begin
    btnCurrentRpt.Caption := 'Click to hide your current report list.';
    for i := 0 to Pred(FExistRPTList.Count) do
    begin
      if Piece(FExistRPTList[i], U, 1) <> '0' then
        lstCurrReports.AddItem(Piece(FExistRPTList[i],'^',2),nil)
      else begin
        lstCurrReports.AddItem('Create Custom Search...',nil);
        lstCurrReports.AddItem('  ',nil);
      end;
    end;
  end else
    btnCurrentRpt.Caption := 'Click to view your current report list.'
end;

procedure TfrmPageSave.txtRptNameKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
    btnFinishClick(Self);
end;

end.
