unit mMultiPatient;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, ORCtrls, Menus, Buttons, ORDtTm;

type
  TfraMultiPatient = class(TFrame)
    pnlLists: TPanel;
    pnlPatients: TPanel;
    Bevel1: TBevel;
    lvwPtSelected: TListView;
    lblPtSelected: TLabel;
    Splitter1: TSplitter;
    tabLists: TTabControl;
    lblAvailable: TLabel;
    lblSelected: TLabel;
    cboShort: TORComboBox;
    btnAdd: TButton;
    grpStatus: TGroupBox;
    radActive: TRadioButton;
    radInactive: TRadioButton;
    radBoth: TRadioButton;
    cboLong: TORComboBox;
    popList: TPopupMenu;
    popDelete: TMenuItem;
    lstSelected: TORListBox;
    btnRemove: TButton;
    pnlDateRange: TPanel;
    lblTimeRange: TLabel;
    btnTimeRange: TSpeedButton;
    lblTimeData: TLabel;
    txtTimeRange: TEdit;
    dtTemp: TORDateBox;
    procedure tabListsChange(Sender: TObject);
    procedure tabListsChanging(Sender: TObject; var AllowChange: Boolean);
    procedure cboLongNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure btnAddClick(Sender: TObject);
    procedure popDeleteClick(Sender: TObject);
    procedure cboLongChange(Sender: TObject);
    procedure cboLongDblClick(Sender: TObject);
    procedure btnTimeRangeClick(Sender: TObject);
    procedure FrameResize(Sender: TObject);
    procedure SetContentForDates(const x: string; var dt: string);
    procedure SetContentForYears(const x: string; var dt: string);
  private
    FAllowRegistry:  Boolean;
    FCurrentSrcType: Char;
    FCurrentSrcName: string;
    function RegistryMode: string;
    procedure AddToSelected(const ACboItem: string);
    procedure UseShortList;
    procedure UseLongList;
    procedure SwitchToClinic;
    procedure SwitchToWard;
    procedure SwitchToTeam;
    procedure SwitchToSpecialty;
    procedure SwitchToProvider;
    procedure SwitchToRegistry;
    function GenerateCaptionTxt(ADateRange: string): string;
  protected
    procedure Loaded; override;
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses ORFn, rDateTime, rPatients, uCohort, fDateRange;

const
  TAB_CLINIC    = 0;
  TAB_WARD      = 1;
  TAB_TEAM      = 2;
  TAB_SPECIALTY = 3;
  TAB_PROVIDER  = 4;
  TAB_REGISTRY  = 5;


procedure TfraMultiPatient.Loaded;
var
  AllowChange: Boolean;
begin
  inherited;
  FAllowRegistry := HasKey('ROR VA HEPC USER');
  if FAllowRegistry then tabLists.Tabs.Add('Registry');
  tabListsChanging(Self, AllowChange);
  case Cohort.LastSourceType of
  SRC_CLINIC    : tabLists.TabIndex := TAB_CLINIC;
  SRC_WARD      : tabLists.TabIndex := TAB_WARD;
  SRC_TEAM      : tabLists.TabIndex := TAB_TEAM;
  SRC_SPECIALTY : tabLists.TabIndex := TAB_SPECIALTY;
  SRC_PROVIDER  : tabLists.TabIndex := TAB_PROVIDER;
  SRC_REGISTRY  : tabLists.TabIndex := TAB_REGISTRY;
  else            tabLists.TabIndex := TAB_REGISTRY;  //TODO: check default list type
  end;
  tabListsChange(Self);
  Cohort.LoadSelectedSources(lstSelected.Items);
end;

procedure TfraMultiPatient.UseShortList;
begin
  cboLong.Visible := False;
  cboShort.Visible := True;
  cboShort.Clear;
end;

procedure TfraMultiPatient.UseLongList;
begin
  cboShort.Visible := False;
  cboLong.Visible := True;
  cboLong.Clear;
  cboLong.InitLongList('');
end;

procedure TfraMultiPatient.SwitchToClinic;
var
  i: integer;
  dt,capTxt: string;
begin
  FCurrentSrcType := SRC_CLINIC;
  FCurrentSrcName := 'Clinic';
  lblAvailable.Caption := 'Type Name or Select from List';
  pnlDateRange.Visible := True;
  UseLongList;
  //jd: show right date range
  for i := 0 to Cohort.SrcList.Count - 1 do
  begin
    if CharAt(Cohort.SrcList[i],1)='c' then
      dt := TSelectedSrc(Cohort.Srclist.Objects[i]).Param;
  end;
  if Length(dt) < 1 then
    dt := 'T:T';
  lblTimeData.Caption := dt;
  capTxt := GenerateCaptionTxt(dt);
  txtTimeRange.Text := capTxt;
end;

procedure TfraMultiPatient.SwitchToProvider;
begin
  FCurrentSrcType := SRC_PROVIDER;
  FCurrentSrcName := 'Provider';
  lblAvailable.Caption := 'Type Name or Select from List';
  UseLongList;
end;

procedure TfraMultiPatient.SwitchToRegistry;
begin
  if not FAllowRegistry then Exit;
  FCurrentSrcType := SRC_REGISTRY;
  FCurrentSrcName := 'Registry';
  lblAvailable.Caption := 'Type Name or Select from List';
  grpStatus.Visible := True;
  UseShortList;
  ListRegistryAll(cboShort.Items);
end;

procedure TfraMultiPatient.SwitchToSpecialty;
begin
  FCurrentSrcType := SRC_SPECIALTY;
  FCurrentSrcName := 'Specialty';
  lblAvailable.Caption := 'Type Name or Select from List (for inpatient only)';
  UseShortList;
  ListSpecialtyAll(cboShort.Items);
end;

procedure TfraMultiPatient.SwitchToTeam;
begin
  FCurrentSrcType := SRC_TEAM;
  FCurrentSrcName := 'Team';
  lblAvailable.Caption := 'Type Name or Select from List';
  UseShortList;
  ListTeamAll(cboShort.Items);
end;

procedure TfraMultiPatient.SwitchToWard;
begin
  FCurrentSrcType := SRC_WARD;
  FCurrentSrcName := 'Ward';
  lblAvailable.Caption := 'Type Name or Select from List';
  UseShortList;
  ListWardAll(cboShort.Items);
end;

procedure TfraMultiPatient.tabListsChanging(Sender: TObject; var AllowChange: Boolean);
begin
  if (tabLists.TabIndex = TAB_CLINIC) then
  begin
    //Cohort.UpdateSoruceValues(lblTimeData.Caption);
    pnlDateRange.Visible := False;
  end;
  if tabLists.TabIndex = TAB_REGISTRY then
  begin
    grpStatus.Visible := False;
  end;
end;

procedure TfraMultiPatient.tabListsChange(Sender: TObject);
begin
  case tabLists.TabIndex of
  TAB_CLINIC    : SwitchToClinic;
  TAB_WARD      : SwitchToWard;
  TAB_TEAM      : SwitchToTeam;
  TAB_SPECIALTY : SwitchToSpecialty;
  TAB_PROVIDER  : SwitchToProvider;
  TAB_REGISTRY  : SwitchToRegistry;
  end;
end;

procedure TfraMultiPatient.cboLongNeedData(Sender: TObject; const StartFrom: String;
  Direction, InsertAt: Integer);
begin
  case tabLists.TabIndex of
  TAB_CLINIC    : cboLong.ForDataUse(SubSetofClinics(  StartFrom, Direction));
  TAB_PROVIDER  : cboLong.ForDataUse(SubSetofProviders(StartFrom, Direction));
  end;
end;

function TfraMultiPatient.RegistryMode: string;
begin
  Result := '3';
  if radActive.Checked then Result := '1';
  if radInactive.Checked then Result := '2';
end;

procedure TfraMultiPatient.AddToSelected(const ACboItem: string);
var
  i: Integer;
  AParam, ASrcID: string;
  redundant: Boolean;
begin
  redundant := False;
  AParam := '';
  if tabLists.TabIndex = TAB_CLINIC   then AParam := lblTimeData.Caption;
  if tabLists.TabIndex = TAB_REGISTRY then AParam := RegistryMode;
  ASrcID := FCurrentSrcType + ':' + Piece(ACboItem, U, 1);
  for i := 0 to Pred(lstSelected.Items.Count) do
    if CompareStr(ASrcID, Piece(lstSelected.Items[i], U, 1)) = 0 then redundant := TRUE;
  if not redundant then
  begin
    lstSelected.Items.Add(ASrcID + U + Piece(ACboItem, U, 2) + U + FCurrentSrcName + U + AParam);
    Cohort.AddPatients(ASrcID, AParam, Piece(ACboItem, U, 2));
  end;
end;

procedure TfraMultiPatient.btnAddClick(Sender: TObject);
var
  ACombo: TORComboBox;
begin
  case tabLists.TabIndex of
  TAB_CLINIC    : ACombo := cboLong;
  TAB_WARD      : ACombo := cboShort;
  TAB_TEAM      : ACombo := cboShort;
  TAB_SPECIALTY : ACombo := cboShort;
  TAB_PROVIDER  : ACombo := cboLong;
  TAB_REGISTRY  : ACombo := cboShort;
  else            ACombo := cboShort;
  end;
  if ACombo.ItemIndex < 0 then Exit;
  AddToSelected(ACombo.Items[ACombo.ItemIndex]);
  ACombo.ItemIndex := -1;
end;

procedure TfraMultiPatient.popDeleteClick(Sender: TObject);
var
  i: Integer;
  //  AnIndex: Integer;
begin
  if lstSelected.ItemIndex < 0 then Exit;
  with lstSelected do
   for i := Items.Count -1 downto 0 do
    if Selected[i] then
    begin
      Cohort.RemovePatients(Piece(lstSelected.Items[i],'^',1));
      Items.Delete(i);
    end;
end;

procedure TfraMultiPatient.cboLongChange(Sender: TObject);
begin
  with Sender as TORComboBox do btnAdd.Default := ItemIndex > -1;
end;

procedure TfraMultiPatient.cboLongDblClick(Sender: TObject);
begin
  btnAddClick(Self);
end;

procedure TfraMultiPatient.btnTimeRangeClick(Sender: TObject);
var
  x: string;
  Values: TStringList;
begin
  x := txtTimeRange.Text;
  Values := TStringList.Create;
  Values.Add(lblTimeData.Caption);
  TfrmDateRange.InvokeEditor(x, Values);
  txtTimeRange.Text := x;
  lblTimeData.Caption := Values[0];
  Values.Free;
end;

procedure TfraMultiPatient.FrameResize(Sender: TObject);
begin
  cboLong.Width := (tabLists.Width - btnAdd.Width - 16) div 2;
  cboShort.Width := cboLong.Width;
  btnAdd.Left := cboLong.Left + cboLong.Width + 2;
  btnRemove.Left := btnAdd.Left;
  lstSelected.Left := btnAdd.Left + btnAdd.Width + 2;
  lstSelected.Width := cboLong.Width;
end;

procedure TfraMultiPatient.SetContentForDates(const x: string; var dt: string);
var
  FMStart, FMStop: TFMDateTime;
  DtStart, DtStop: TDateTime;
begin
  if Piece(x,':',1)<>'0' then
  begin
    FMStart := StrToFMDateTime(Piece(x, ':', 1));
    DtStart := FMDateTimeToDateTime(FMStart);
  end else
    DtStart := 0;
  if Piece(x,':',2)<>'0' then
  begin
    FMStop  := StrToFMDateTime(Piece(x, ':', 2));
    DtStop := FMDateTimeToDateTime(FMStop);
  end else
    DtStop := 0;
  if (DtStart=0) and (DtStop=0) then
    dt := 'any date'
  else if (DtStart=0) and (DtStop <> 0) then
    dt := 'to ' + DateToStr(DtStop)
  else if (DtStop=0) and (DtStart <> 0) then
    dt := 'from ' + DateToStr(DtStart);
end;

function TfraMultiPatient.GenerateCaptionTxt(ADateRange: string): string;
var
  Captxt: string;
begin
  Captxt := 'during Today';
  if      ADateRange = 'T:T'    then Captxt := 'during Today'
  else if ADateRange = 'T-1:T'  then Captxt := 'during the Past 24 Hours'
  else if ADateRange = 'T-7:T'  then Captxt := 'during the Past Week'
  else if ADateRange = 'T-30:T' then Captxt := 'during the Past Month'
  else if ADateRange = 'YFQ'    then Captxt := 'during the Current Quarter'
  else if ADateRange = 'YF'     then Captxt := 'during the Current Year'
  else if CharAt(ADateRange, 1) = 'Y' then SetContentForYears(AdateRange, Captxt)
  else SetContentForDates(ADateRange, Captxt);
  Result := Captxt;
end;

procedure TfraMultiPatient.SetContentForYears(const x: string;
  var dt: string);
const
  TX_CY_FIRST  = '1st Quarter (Jan-Mar)';
  TX_CY_SECOND = '2nd Quarter (Apr-Jun)';
  TX_CY_THIRD  = '3rd Quarter (Jul-Sep)';
  TX_CY_FOURTH = '4th Quarter (Oct-Dec)';
  DU           = 'during ';
var
  Offset, Qtr:  Integer;
  CurrentYear,CurrentFiscal,CurrentMonth: integer;
  QtrArray: array [1..4] of string;
  theYear,theQuarter: string;
begin
  theYear := '';
  theQuarter := '';
  QtrArray[1] := TX_CY_FIRST;
  QtrArray[2] := TX_CY_SECOND;
  QtrArray[3] := TX_CY_THIRD;
  QtrArray[4] := TX_CY_FOURTH;
  CurrentMonth := (Trunc(FMToday) mod 10000) div 100;   // extract month from Fileman date
  CurrentYear  := Trunc(FMToday / 10000) + 1700;        // extract year from Fileman date
  if CurrentMonth > 9
    then CurrentFiscal := CurrentYear + 1
    else CurrentFiscal := CurrentYear;
  Offset := 0;
  if Pos('+', x) > 0 then Offset :=   StrToIntDef(Piece(Piece(x, '+', 2), 'Q', 1), 0);
  if Pos('-', x) > 0 then Offset := -(StrToIntDef(Piece(Piece(x, '-', 2), 'Q', 1), 0));
  if x[2] = 'C' then theYear := IntToStr(CurrentYear + Offset);
  if x[2] = 'F' then theYear := 'FY'+IntToStr(CurrentFiscal + Offset);
  Qtr := StrToIntDef(Piece(x, 'Q', 2), 0);
  if (Qtr > 0) and (Qtr < 5) then
    theQuarter := QtrArray[Qtr];
  dt := DU +  theQuarter + ' ' + theYear;
end;

end.
