unit mQuery;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  Menus, ExtCtrls, uReport;

type
  TfraQuery = class(TFrame)
    lblSize: TLabel;
    sbxQuery: TScrollBox;
    shpFocus: TShape;
    procedure Other1Click(Sender: TObject);
    procedure sbxQueryEnter(Sender: TObject);
  private
    procedure ClearLabels;
    function CreateStaticLabelAt(ALeft, ATop, ATag: Integer; const LabelText: string): Integer;
    function CreateLinkedLabelAt(ALeft, ATop, ATag: Integer; AQueryText: TQueryText): Integer;
    procedure lblLinkClick(Sender: TObject);
    procedure lblLinkEnter(Sender: TObject);
    procedure lblLinkExit(Sender: TObject);
    procedure lblLinkKeyPress(Sender: TObject; var Key: Char);
  public
    procedure BuildQuery;
  end;

  TFocusLabel = class(TStaticText)
  public
    property OnEnter;
    property OnExit;
    property OnKeyPress;
  end;

implementation

uses ORFn, fDateRange, fMultiList, fMultiCheck, fFreeText, fPtList, fPageReports;

{$R *.DFM}

const
  PIX_VERT_SPACE  = 5;
  PIX_VERT_MARGIN = 2;
  PIX_LEFT_MARGIN = 2;
  PIX_LEFT_INDENT = 15;

{ TfraQuery }

procedure TfraQuery.ClearLabels;
var
  i: Integer;
  AControl: TControl;
begin
  for i := Pred(sbxQuery.ControlCount) downto 0 do
  begin
    AControl := sbxQuery.Controls[i];
    if AControl is TLabel       then sbxQuery.RemoveControl(AControl);
    if AControl is TFocusLabel  then sbxQuery.RemoveControl(AControl);
  end;
end;

function TfraQuery.CreateStaticLabelAt(ALeft, ATop, ATag: Integer; const LabelText: string): Integer;
var
  ALabel: TLabel;
begin
  ALabel := TLabel.Create(Self);
  ALabel.Parent := sbxQuery;
  ALabel.Tag := ATag;
  ALabel.Caption := LabelText + ' ';
  ALabel.Top := ATop;
  ALabel.Left := ALeft;
  Result := ALeft + ALabel.Width;
end;

function TfraQuery.CreateLinkedLabelAt(ALeft, ATop, ATag: Integer; AQueryText: TQueryText): Integer;
var
  ALabel: TFocusLabel;
begin
  ALabel := TFocusLabel.Create(Self);
  ALabel.Parent := sbxQuery;
  ALabel.Tag := ATag;
  ALabel.Caption := AQueryText.Caption + ' ';
  ALabel.Top := ATop;
  ALabel.Left := ALeft;
  ALabel.TabStop := True;
  if AQueryText.Values.Count > 0
    then ALabel.Font.Color := clPurple
    else ALabel.Font.Color := clBlue;
  ALabel.Font.Style := [fsUnderline];
  ALabel.OnClick    := lblLinkClick;
  ALabel.OnEnter    := lblLinkEnter;
  ALabel.OnExit     := lblLinkExit;
  ALabel.OnKeyPress := lblLinkKeyPress;
  Result := ALeft + ALabel.Width;
end;

procedure TfraQuery.lblLinkClick(Sender: TObject);
var
  x: string;

  function DoAbnormalStartDateCheck: boolean;
  var
    i: integer;
  begin
    Result := False;
    if AnsiCompareText(Report.Name,'RPT ABNORMAL RESULTS') = 0 then
    begin
      Result := True;
      Exit;
    end;
    i := 0;
    while Report.QueryText[i] <> nil do
    begin
      if AnsiCompareText(Report.QueryText[i].ConstraintName,'Order.Abnormal')= 0 then
      begin
        Result := True;
        Exit;
      end;
      i := i + 1;
    end;
  end;
begin
  shpFocus.Visible := False;
  with Report.QueryText[TStaticText(Sender).Tag] do
  begin
    x := Caption;
    case EditorId of
     1: TfrmPtList.InvokeEditor(x, Values);                     //Patient Selection (Multiple)
     2: begin
          if DoAbnormalStartDateCheck then
            TfrmDateRange.InvokeEditor(x, Values,True)
          else
            TfrmDateRange.InvokeEditor(x, Values);
        end;
     3: TfrmMultiList.InvokeEditor(x, Values, ITM_CLINIC);      //Clinic List (Multiple)
     4: TfrmMultiList.InvokeEditor(x, Values, ITM_SERVICE);     //Display Group (Mulitple)
     5: TfrmMultiList.InvokeEditor(x, Values, ITM_ORDERABLE);   //Orderable Item (Multiple)
     6: TfrmMultiCheck.InvokeEditor(x, Values, STS_ORDER);      //Order Status (Multiple)
     7: TfrmMultiCheck.InvokeEditor(x, Values, STS_ORDER_SIGN); //Order Sign Status (Multiple)
     8: TfrmMultiList.InvokeEditor(x, Values, ITM_PROVIDER);    //Person (Multiple)
     9: TfrmFreeText.InvokeEditor(x, Values);                   //Text Line
    10: TfrmMultiList.InvokeEditor(x, Values, ITM_DOCUMENT);    //Document Title (multiple)
    11: TfrmMultiCheck.InvokeEditor(x, Values, STS_DOCUMENT);   //Document Status (multiple)
    13: TfrmMultiCheck.InvokeEditor(x, Values, STS_CLASS);      //Document Status (multiple)
    14: TfrmMultiList.InvokeEditor(x, Values, ITM_CONSULT);     //Consult Services
    15: TfrmMultiList.InvokeEditor(x, Values, ITM_PROCEDURE);   //Procedures
    else ShowMessage('Missing Editor Link');
    end; {case}
    Caption := x;
    BuildQuery;
  end; {with}
end;

procedure TfraQuery.lblLinkEnter(Sender: TObject);
begin
  with Sender as TFocusLabel do
  begin
    shpFocus.SetBounds(Left-1, Top-1, Width+2, Height+2);
    shpFocus.Visible := True;
    shpFocus.BringToFront;
  end;
end;

procedure TfraQuery.lblLinkExit(Sender: TObject);
begin
  shpFocus.Visible := False;
end;

procedure TfraQuery.lblLinkKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #32 then lblLinkClick(Sender);
end;

procedure TfraQuery.BuildQuery;
var
  i, ATop, ALeft, LastCriterion, LineCount: Integer;

  procedure ReDoDateRangeCaption(AQueryText: TQueryText);
  var
    j: integer;
  begin
    if AnsiCompareText(AQueryText.ConstraintName,'Search.DateRange')=0 then
    begin
      if AQueryText.Values.Count = 0 then Exit;
      if Piece(AQueryText.Values[0],':',2)='0' then
      begin
        j := Pos('through',AQueryText.Caption);
        if j > 1 then
          AQueryText.Caption := Copy(AQueryText.Caption,1,j-1);
      end
      else if Piece(AQueryText.Values[0],':',1)='0' then
      begin
        j := Pos('from',AQueryText.Caption);
        if j > 0 then
          AQueryText.Caption :=Trim(Copy(AQueryText.Caption,j+length('from'),Length(AQueryText.Caption)));
      end;
    end;
  end;

begin
  ClearLabels;
  LastCriterion := 0;
  LineCount := 0;
  ATop  := PIX_VERT_MARGIN - lblSize.Height;
  ALeft := PIX_LEFT_MARGIN;
  i := 0;
  repeat
    if Report.QueryText[i].CriterionTag <> LastCriterion then
    begin
      LastCriterion := Report.QueryText[i].CriterionTag;
      ATop := ATop + lblSize.Height + PIX_VERT_SPACE;
      ALeft := PIX_LEFT_MARGIN;
      if (LastCriterion div 10000) mod 10 = 0
        then LineCount := 0
        else ALeft := PIX_LEFT_INDENT;
      if AnsiCompareText(Report.QueryText[i].ConstraintName,'Search.DateRange')=0 then
        if Pos('Searching Reports',Report.QueryText[i].Caption) = 0  then
          ALeft := PIX_LEFT_INDENT;
      Inc(LineCount);
      if LineCount > 2 then ALeft := CreateStaticLabelAt(ALeft, ATop, 0, 'and');
    end;
    ReDoDateRangeCaption(Report.QueryText[i]);    
    if (Report.QueryText[i].ConstraintName = '') or
       (Report.QueryText[i].EditorId = EDITOR_NONE)
      then ALeft := CreateStaticLabelAt(ALeft, ATop, i, Report.QueryText[i].Caption)
      else ALeft := CreateLinkedLabelAt(ALeft, ATop, i, Report.QueryText[i]);
    Inc(i);
  until Report.QueryText[i] = nil;
end;

procedure TfraQuery.Other1Click(Sender: TObject);
begin
  lblLinkClick(TPopupMenu(TMenuItem(Sender).GetParentMenu).PopupComponent);
end;

procedure TfraQuery.sbxQueryEnter(Sender: TObject);
// When sbxQuery gets focus, change focus to the first TFocusLabel on sbxQuery.
var
  i: integer;
  FocusLabelFound: Boolean;

begin
  FocusLabelFound := FALSE;
  if sbxQuery.ControlCount > 0 then
     begin
       for i := 0 to (sbxQuery.ControlCount - 1) do
         begin
           if sbxQuery.Controls[i] is TFocusLabel then
              begin
                TFocusLabel(sbxQuery.Controls[i]).SetFocus;
                FocusLabelFound := TRUE;
                break;
              end;
         end;
       if not FocusLabelFound then
         TfrmPageReports(TPanel(self.Parent).Parent).btnEdit.SetFocus;
     end
  else
    TfrmPageReports(TPanel(self.Parent).Parent).btnEdit.SetFocus;
end;

end.
